# coding=utf-8
# vim: set tabstop=2 shiftwidth=2:
# Copyright 2023 SJTU X-Lance Lab
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# 
# Revised by Danyang Zhang @X-Lance based on
# 
# Copyright 2021 DeepMind Technologies Limited.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Determines if the current app screen matches an expected app screen."""

import enum
import re
from typing import Callable, Optional, Sequence
from typing import List, Pattern, Tuple

from absl import logging

from android_env.components import adb_controller as adb_control
from android_env.proto import task_pb2

import lxml.etree

class DumpsysNode():
  """A node in a dumpsys tree."""

  def __init__(self, data: Optional[str] = None):
    self._children = []
    self._data = data

  @property
  def data(self) -> str:
    return self._data

  @property
  def children(self) -> List['DumpsysNode']:
    return self._children

  def find_child(self,
                 predicate: Callable[['DumpsysNode'], bool],
                 max_levels: int = 0) -> Optional['DumpsysNode']:
    """Returns the first direct child that matches `predicate`, None otherwise.

    Args:
      predicate: Function-like that accepts a DumpsysNode and returns boolean.
      max_levels: Maximum number of levels down the tree to search for a child.
        If non-positive, only direct children will be searched for.

    Returns:
      A DumpsysNode or None.
    """
    if not self.children:
      return None

    try:
      return next(x for x in self.children if predicate(x))
    except StopIteration:
      logging.info('Failed to find child. max_levels: %i.', max_levels)
      # Search children.
      if max_levels:
        for child in self.children:
          child_result = child.find_child(predicate, max_levels - 1)
          if child_result is not None:
            return child_result

      return None

  def __repr__(self):
    return self._data

  def print_tree(self, indent: int = 2):
    """Prints this tree in logging.info()."""
    logging.info(' ' * indent + self.data)
    for c in self.children:
      c.print_tree(indent + 2)


def build_tree_from_dumpsys_output(dumpsys_output: str) -> DumpsysNode:
  """Constructs a tree from a dumpsys string output.

  Args:
    dumpsys_output: string Verbatim output from adb dumpsys. The expected format
      is a list where each line is a node and the indentation marks the
      relationship with its parent or sibling.

  Returns:
    DumpsysNode The root of the tree.
  """
  lines = dumpsys_output.split('\n')  # Split by lines.
  lines = [x.rstrip(' \r') for x in lines]
  lines = [x for x in lines if len(x)]  # Remove empty lines.

  root = DumpsysNode('___root___')  # The root of all nodes.
  parents_stack = [root]
  for line in lines:
    stripped_line = line.lstrip(' ')
    indent = len(line) - len(stripped_line)  # Number of indent spaces.
    new_node = DumpsysNode(stripped_line)  # Create a node without indentation.

    parent = parents_stack.pop()
    if parent.data == '___root___':  # The root is an exception for indentation.
      parent_indent = -2
    else:
      parent_indent = (len(parents_stack) - 1) * 2

    if indent == parent_indent:  # `new_node` is a sibiling.
      parent = parents_stack.pop()
    elif indent < parent_indent:  # Indentation reduced (i.e. a block finished)
      num_levels = (indent // 2) + 1
      parents_stack = parents_stack[:num_levels]
      parent = parents_stack.pop()
    elif indent > parent_indent:  # `new_node` is a child.
      pass  # No need to change the current parent.

    parent.children.append(new_node)
    parents_stack.append(parent)
    parents_stack.append(new_node)

  return root


def matches_path(dumpsys_activity_output: str,
                 expected_view_hierarchy_path: Sequence[Pattern[str]],
                 max_levels: int = 0) -> bool:
  """Returns True if the current dumpsys output matches the expected path.

  Args:
    dumpsys_activity_output: The output of running `dumpsys activity ...`.
    expected_view_hierarchy_path: [regex] A list of regular expressions to be
      tested at each level of the tree.
    max_levels: How many levels to search from root for View Hierarchy.

  Returns:
    True if the dumpsys tree contains one path that matches all regexes.
  """
  root = build_tree_from_dumpsys_output(dumpsys_activity_output)

  # Find the View Hierarchy.
  view_hierarchy = root.find_child(
      lambda x: x.data.startswith('View Hierarchy'), max_levels)
  if view_hierarchy is None:
    logging.error(
        'view_hierarchy is None. Dumpsys activity output: %s. tree: %r',
        str(dumpsys_activity_output), root.print_tree())
    logging.error('Tree root: %s', str(root))
    return None

  current_node = view_hierarchy
  for i, regex in enumerate(expected_view_hierarchy_path):

    def regex_predicate(node, expr=regex):
      matches = expr.match(node.data)
      return matches is not None

    child = current_node.find_child(regex_predicate)
    if child is None:
      logging.error('Mismatched regex (%i, %s). current_node: %s', i,
                    regex.pattern, current_node)
      logging.error('Dumpsys activity output: %s', str(dumpsys_activity_output))
      logging.error('Tree root: %s', str(root))
      return None
    else:
      current_node = child
  return True

def find_children(node: lxml.etree.Element,
    class_regex: str, id_regex: str) -> List[lxml.etree.Element]:
  #  function `find_children` {{{ # 
  """
  node - lxml.etree.Element
  class_regex - str
  id_regex - str

  return list of lxml.etree.Element
  """

  children = []
  for ch in node.iterdescendants():
    #if class_regex.match(ch.get("class")) is not None and\
        #id_regex.match(ch.get("resource-id")) is not None:
    if class_regex==ch.get("class") and\
        (id_regex=="" or id_regex==ch.get("resource-id")):
      children.append(ch)
  return children
  #  }}} function `find_children` # 

_separator_pattern = re.compile(r"(?<!\\)@")
_fake_separator_pattern = re.compile(r"\\@")
def match_path2(node: lxml.etree.Element, vh_path: List[str])\
    -> Tuple[bool, Optional[lxml.etree.Element]]:
  #  function `match_path2` {{{ # 
  """
  node - lxml.etree.Element
  vh_path - list of str
  
  return
  - bool
  - lxml.etree.Element or None
  """

  if len(vh_path)==0:
    return True, node

  head = vh_path[0]
  tail = vh_path[1:]

  patterns = _separator_pattern.split(head, maxsplit=1)
  class_pattern_str = _fake_separator_pattern.sub("@", patterns[0])
  #class_regex = re.compile(class_regex)
  class_regex = class_pattern_str
  id_pattern_str = _fake_separator_pattern.sub("@", patterns[1]) if len(patterns)>=2\
      else ""
      #else r".*"
  #id_regex = re.compile(id_pattern_str)
  id_regex = id_pattern_str

  matched_children = find_children(node, class_regex, id_regex)
  for ch in matched_children:
    matches, leaf = match_path2(ch, tail)
    if matches:
      return True, leaf
  return False, None
  #  }}} function `match_path2` # 

class AppScreenChecker():
  """Checks that the current app screen matches an expected screen."""

  bbox_regex = re.compile(r"\[(?P<left>\d+),(?P<top>\d+)\]\[(?P<right>\d+),(?P<bottom>\d+)\]")

  class Outcome(enum.IntEnum):
    """Possible return vales from checking the current app screen."""
    # The current app screen matches the expected app screen.
    SUCCESS = 0
    # There's no activity to check.
    EMPTY_EXPECTED_ACTIVITY = 1
    # We were unable to determine the current activity.
    FAILED_ACTIVITY_EXTRACTION = 2
    # The current activity does not match the expected activity.
    UNEXPECTED_ACTIVITY = 3
    # The current view hierarchy does not match the expected view hierarchy.
    UNEXPECTED_VIEW_HIERARCHY = 4

  def __init__(self,
               adb_controller: adb_control.AdbController,
               expected_app_screen: task_pb2.AppScreen):

    self._adb_controller = adb_controller
    self._expected_activity = expected_app_screen.activity
    self._expected_view_hierarchy_path = [
        re.compile(regex) for regex in expected_app_screen.view_hierarchy_path
    ]

    self._event_listeners = [] # zdy

  # Return type is AppScreenChecker.Outcome, but pytype doesn't understand that.
  def matches_current_app_screen(self) -> enum.IntEnum:
    """Determines whether the current app screen matches `expected_app_screen`."""
    if not self._expected_activity:
      return AppScreenChecker.Outcome.EMPTY_EXPECTED_ACTIVITY

    # Check if we are still on the expected Activity.
    current_activity = self._adb_controller.get_current_activity()
    if current_activity is None:
      return AppScreenChecker.Outcome.FAILED_ACTIVITY_EXTRACTION

    if current_activity != self._expected_activity:
      logging.error('current_activity: %s,  expected_activity: %s',
                    current_activity, self._expected_activity)
      return AppScreenChecker.Outcome.UNEXPECTED_ACTIVITY

    # Extract just the package name from the full activity name.
    package_name = self._expected_activity.split('/')[0]

    # Check if we are in the expected view hierarchy path.
    if self._expected_view_hierarchy_path:
      dumpsys_activity_output = self._adb_controller.get_activity_dumpsys(
          package_name)
      if dumpsys_activity_output:
        if not matches_path(
            dumpsys_activity_output,
            self._expected_view_hierarchy_path,
            max_levels=3):
          return AppScreenChecker.Outcome.UNEXPECTED_VIEW_HIERARCHY

    return AppScreenChecker.Outcome.SUCCESS

  #  For VH Events Listening {{{ # 
  def add_event_listeners(self, *event_listeners):
    #  method `add_event_listeners` {{{ # 
    """
    event_listeners - list of event_listeners.ViewHierarchyEvent
    """

    self._event_listeners += event_listeners
    #  }}} method `add_event_listeners` # 

  def match_events(self, lock):
    #  method `match_events` {{{ # 
    """
    lock - threading.Lock
    """

    view_hierarchy = self._adb_controller.get_view_hierarchy()
    if view_hierarchy is None:
      return

    for evnt in self._event_listeners:
      # 1. match the path
      matches, leaf_node = match_path2(view_hierarchy, evnt.path)
      if not matches:
        continue

      # 2. match the property value
      values = []
      for prpt in evnt.property_names:
        if prpt in ["left", "top", "right", "bottom"]:
          matches = AppScreenChecker.bbox_regex.fullmatch(leaf_node.get("bounds"))
          normalization = self._adb_controller.get_screen_dimensions()[1]\
              if prpt[0]=="l" or prpt[0]=="r"\
              else self._adb_controller.get_screen_dimensions()[0]
          values.append(float(matches[prpt])/normalization)
        else:
          values.append(leaf_node.get(prpt))
      with lock:
        evnt.set(values)
    #  }}} method `match_events` # 
  #  }}} For VH Events Listening # 



# coding=utf-8
# vim: set tabstop=2 shiftwidth=2:
# Copyright 2023 SJTU X-Lance Lab
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# 
# Revised by Danyang Zhang @X-Lance based on
# 
# Copyright 2021 DeepMind Technologies Limited.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""A class to manage and control an external ADB process."""

import os
import pathlib
import re
import subprocess
import sys
import threading
import time
from typing import List, Optional, Sequence, Tuple
import lxml.etree

from absl import logging
from android_env.components import errors
from android_env.proto import adb_pb2
import pexpect

_MAX_INIT_RETRIES = 20
_INIT_RETRY_SLEEP_SEC = 2.0

_DEFAULT_TIMEOUT_SECONDS = 120.0


class AdbController():
  """Manages communication with adb."""

  def __init__(self,
               device_name: str = '',
               adb_path: str = 'adb',
               adb_server_port: int = 5037,
               prompt_regex: str = r'generic_x86:/ [$#]',
               default_timeout: float = _DEFAULT_TIMEOUT_SECONDS,
               #frida_server: Optional[str] = None,
               frida: Optional[str] = None,
               frida_script: Optional[str] = None):
    """Instantiates an AdbController object.

    Args:
      device_name: Name of the device to communicate with.
      adb_path: Path to the adb binary.
      adb_server_port: Port for adb server.
      prompt_regex: Shell prompt for pexpect in ADB controller.
      default_timeout: Default timeout in seconds.
      #frida_server: Path to the frida_server executable on the Android device.
      frida: Path to the frida binary executable on the host machine
      frida_script: Path to the frida script on the host machine
    """

    self._device_name = device_name
    self._adb_path = adb_path
    self._adb_server_port = str(adb_server_port)
    self._prompt = prompt_regex
    self._default_timeout = default_timeout

    self._frida: Optional[str] = frida
    self._frida_script: Optional[str] = frida_script
    if self._frida and self._frida_script:
      self._frida_processes: Dict[str, subprocess.Popen] = {}

    self._platform_sys = sys.platform
    self._execute_command_lock = threading.Lock()
    self._adb_shell = None
    self._shell_is_ready = False
    # Unset problematic environment variables. ADB commands will fail if these
    # are set. They are normally exported by AndroidStudio.
    if 'ANDROID_HOME' in os.environ:
      del os.environ['ANDROID_HOME']
    if 'ANDROID_ADB_SERVER_PORT' in os.environ:
      del os.environ['ANDROID_ADB_SERVER_PORT']

  def command_prefix(self) -> List[str]:
    """The command for instantiating an adb client to this server."""
    command_prefix = [self._adb_path, '-P', self._adb_server_port]
    if self._device_name:
      command_prefix.extend(['-s', self._device_name])
    return command_prefix

  def init_server(self, timeout: Optional[float] = None):
    """Initialize the ADB server deamon on the given port.

    This function should be called immediately after initializing the first
    adb_controller, and before launching the simulator.

    Args:
      timeout: A timeout to use for this operation. If not set the default
        timeout set on the constructor will be used.
    """
    # Make an initial device-independent call to ADB to start the deamon.
    device_name_tmp = self._device_name
    self._device_name = ''
    self._execute_command(['devices'], timeout=timeout)
    time.sleep(0.2)
    # Subsequent calls will use the device name.
    self._device_name = device_name_tmp

  def get_root(self, timeout: Optional[float] = None):
    """
    Gets the root priviledge.

    timeout - float or None
    """

    self._execute_command(["root"], timeout=timeout)
    time.sleep(0.2)
    #if self._prompt.endswith(r"\$"):
      #self._prompt = self._prompt[:-2] + "#"
    #self._init_shell()

  def init_frida_server(self, frida_server: str, timeout: Optional[float] = None):
    """
    Launch the frida server.

    frida_server - str
    timeout - float or None
    """

    #self._execute_command(["shell", frida_server, "&"])
    #time.sleep(0.2)
    self._frida_server = subprocess.Popen(
        [
          "adb",
          "shell",
          frida_server
        ],
        stdin=subprocess.DEVNULL)

  def close(self) -> None:
    """Closes internal threads and processes."""
    logging.info('Closing ADB controller...')
    if self._adb_shell is not None:
      logging.info('Killing ADB shell')
      self._adb_shell.close(force=True)
      self._adb_shell = None
      self._shell_is_ready = False
    logging.info('Done closing ADB controller.')
    if hasattr(self, "_frida_processes"):
      for prcss in self._frida_processes.values():
        prcss.terminate()
      self._frida_processes.clear()
    if hasattr(self, "_frida_server"):
      self._frida_server.terminate()

  def _execute_command(
      self,
      args: List[str],
      timeout: Optional[float] = None) -> Optional[bytes]:
    """Executes an adb command.

    Args:
      args: A list of strings representing each adb argument.
          For example: ['install', '/my/app.apk']
      timeout: A timeout to use for this operation. If not set the default
        timeout set on the constructor will be used.

    Returns:
      The output of running such command as a string, None if it fails.
    """
    # The lock here prevents commands from multiple threads from messing up the
    # output from this AdbController object.
    with self._execute_command_lock:
      if self._platform_sys == 'win32':
        adb_output = self._execute_normal_command(args, timeout=timeout)
      elif args and args[0] == 'shell':
        adb_output = self._execute_shell_command(args[1:], timeout=timeout)
      else:
        adb_output = self._execute_normal_command(args, timeout=timeout)
    logging.debug('ADB output: %s', adb_output)
    return adb_output

  def _execute_normal_command(
      self,
      args: List[str],
      timeout: Optional[float] = None) -> Optional[bytes]:
    """Executes `adb args` and returns its output."""

    timeout = self._resolve_timeout(timeout)
    command = self.command_prefix() + args
    logging.info('Executing ADB command: %s', command)

    try:
      cmd_output = subprocess.check_output(
          command, stderr=subprocess.STDOUT, timeout=timeout)
      logging.info('Done executing ADB command: %s', command)
      return cmd_output
    except (subprocess.CalledProcessError, subprocess.TimeoutExpired) as error:
      logging.exception('Failed to execute ADB command %s', command)
      logging.error('Error output: %r', error.output)
      raise error

  def _execute_shell_command(
      self,
      args: List[str],
      timeout: Optional[float] = None,
      max_num_retries: int = 3) -> Optional[bytes]:
    """Execute shell command."""

    timeout = self._resolve_timeout(timeout)
    if not self._shell_is_ready:
      self._init_shell(timeout=timeout)
    shell_args = ' '.join(args)
    logging.info('Executing ADB shell command: %s', shell_args)

    num_tries = 0
    while num_tries < max_num_retries:
      num_tries += 1
      try:
        self._adb_shell.sendline(shell_args)
        self._adb_shell.expect(self._prompt, timeout=timeout)
        logging.info('Done executing ADB shell command: %s', shell_args)
        output = self._adb_shell.before.partition('\n'.encode('utf-8'))[2]
        return output
      except (pexpect.exceptions.EOF, pexpect.exceptions.TIMEOUT):
        logging.exception('Shell command failed. Reinitializing the shell.')
        logging.warning('self._adb_shell.before: %r', self._adb_shell.before)
        self._init_shell(timeout=timeout)

    logging.exception('Reinitializing the shell did not solve the issue.')
    raise errors.AdbControllerPexpectError()

  def _init_shell(self, timeout: Optional[float] = None) -> None:
    """Starts an ADB shell process.

    Args:
      timeout: A timeout to use for this operation. If not set the default
        timeout set on the constructor will be used.

    Raises:
        errors.AdbControllerShellInitError when adb shell cannot be initialized.
    """

    timeout = self._resolve_timeout(timeout)
    command = ' '.join(self.command_prefix() + ['shell'])
    logging.info('Initialising ADB shell with command: %s', command)

    num_tries = 0
    while num_tries < _MAX_INIT_RETRIES:
      num_tries += 1
      try:
        logging.info('Spawning ADB shell...')
        self._adb_shell = pexpect.spawn(command, use_poll=True, timeout=timeout)
        # Setting this to None prevents a 50ms wait for each sendline.
        self._adb_shell.delaybeforesend = None
        self._adb_shell.delayafterread = None
        logging.info('Done spawning ADB shell. Consuming first prompt...')
        self._adb_shell.expect(self._prompt, timeout=timeout)
        logging.info('Done consuming first prompt.')
        self._shell_is_ready = True
        return
      except (pexpect.ExceptionPexpect, ValueError) as e:
        logging.exception(e)
        logging.error('self._adb_shell.before: %r', self._adb_shell.before)
        logging.error('Could not start ADB shell. Try %r of %r.',
                      num_tries, _MAX_INIT_RETRIES)
        time.sleep(_INIT_RETRY_SLEEP_SEC)

    raise errors.AdbControllerShellInitError(
        'Failed to start ADB shell. Max number of retries reached.')

  def _wait_for_device(self,
                       max_tries: int = 20,
                       sleep_time: float = 1.0,
                       timeout: Optional[float] = None) -> None:
    """Waits for the device to be ready.

    Args:
      max_tries: Number of times to check if device is ready.
      sleep_time: Sleep time between checks, in seconds.
      timeout: A timeout to use for this operation. If not set the default
        timeout set on the constructor will be used.

    Returns:
      True if the device is ready, False if the device timed out.
    Raises:
      errors.AdbControllerDeviceTimeoutError when the device is not ready after
        exhausting `max_tries`.
    """
    num_tries = 0
    while num_tries < max_tries:
      ready = self._check_device_is_ready(timeout=timeout)
      if ready:
        logging.info('Device is ready.')
        return
      time.sleep(sleep_time)
      logging.error('Device is not ready.')
    raise errors.AdbControllerDeviceTimeoutError('Device timed out.')

  def _check_device_is_ready(self, timeout: Optional[float] = None) -> bool:
    """Checks if the device is ready."""
    required_services = ['window', 'package', 'input', 'display']
    for service in required_services:
      check_output = self._execute_command(
          ['shell', 'service', 'check', service],
          timeout=timeout)
      if not check_output:
        logging.error('Check for service "%s" failed.', service)
        return False
      check_output = check_output.decode('utf-8').strip()
      if check_output != f'Service {service}: found':
        logging.error(check_output)
        return False
    return True

  def _resolve_timeout(self, timeout: Optional[float]) -> float:
    """Returns the correct timeout to be used for external calls."""
    return self._default_timeout if timeout is None else timeout

  # ===== SPECIFIC COMMANDS =====

  def install_binary(self,
                     src: str,
                     dest_dir: str,
                     timeout: Optional[float] = None):
    """Installs the specified binary on the device."""
    self._execute_command(
        ['shell', 'su', '0', 'mkdir', '-p', dest_dir],
        timeout=timeout)
    self._execute_command(
        ['shell', 'su', '0', 'chown', '-R', 'shell:', dest_dir],
        timeout=timeout)
    bin_name = pathlib.PurePath(src).name
    dest = pathlib.PurePath(dest_dir) / bin_name
    self.push_file(src, str(dest), timeout=timeout)

  def install_apk(self,
                  local_apk_path: str,
                  timeout: Optional[float] = None) -> None:
    """Installs an app given a `local_apk_path` in the filesystem.

    This function checks that `local_apk_path` exists in the file system, and
    will raise an exception in case it doesn't.

    Args:
      local_apk_path: Path to .apk file in the local filesystem.
      timeout: A timeout to use for this operation. If not set the default
        timeout set on the constructor will be used.
    """
    assert os.path.exists(local_apk_path), (
        'Could not find local_apk_path :%r' % local_apk_path)
    self._execute_command(['install', '-r', '-t', '-g', local_apk_path],
                          timeout=timeout)

  def uninstall_package(self,
                        package_name: str,
                        timeout: Optional[float] = None) -> None:
    """Uninstalls an app if it is already installed."""
    if self.is_package_installed(package_name, timeout):
      self._execute_command(['uninstall', package_name], timeout=timeout)

  def is_package_installed(self,
                           package_name: str,
                           timeout: Optional[float] = None) -> bool:
    """Checks that the given package is installed."""
    packages = self._execute_command(
        ['shell', 'pm', 'list', 'packages'], timeout=timeout)
    if not packages:
      return False
    packages = packages.decode('utf-8').split()
    # Remove 'package:' prefix for each package.
    packages = [pkg[8:] for pkg in packages if pkg[:8] == 'package:']
    logging.info('Installed packages: %r', packages)
    if package_name in packages:
      logging.info('Package %s found.', package_name)
      return True
    return False

  def start_activity(self,
                     full_activity: str,
                     extra_args: Optional[List[str]],
                     timeout: Optional[float] = None):
    if self._frida and self._frida_script:
      package_name = full_activity.split("/", maxsplit=1)[0]
      self._frida_processes[package_name] = subprocess.Popen(
          [
            self._frida,
            "--no-pause", "-U",
            "-l", self._frida_script,
            "-f", package_name
          ],
          stdin=subprocess.DEVNULL)
      return

    if extra_args is None:
      extra_args = []
    self._execute_command(
        ['shell', 'am', 'start', '-S', '-n', full_activity] + extra_args,
        timeout=timeout)

  def start_intent(self,
                   action: str,
                   data_uri: str,
                   package_name: str,
                   timeout: Optional[float] = None):
    self._execute_command(
        ['shell', 'am', 'start', '-a', action, '-d', data_uri, package_name],
        timeout=timeout)

  def start_accessibility_service(self,
                                  accessibility_service_full_name,
                                  timeout: Optional[float] = None):
    self._execute_command(
        ['shell', 'settings', 'put', 'secure', 'enabled_accessibility_services',
         accessibility_service_full_name], timeout=timeout)

  def broadcast(self,
                receiver: str,
                action: str,
                extra_args: Optional[List[str]],
                timeout: Optional[float] = None):
    if extra_args is None:
      extra_args = []
    self._execute_command(
        ['shell', 'am', 'broadcast', '-n', receiver, '-a', action] + extra_args,
        timeout=timeout)

  def setprop(self,
              prop_name: str,
              value: str,
              timeout: Optional[float] = None):
    self._execute_command(
        ['shell', 'setprop', prop_name, value], timeout=timeout)

  def push_file(self, src: str, dest: str, timeout: Optional[float] = None):
    self._execute_command(
        ['push', src, dest], timeout=timeout)

  def force_stop(self, package: str, timeout: Optional[float] = None):
    if hasattr(self, "_frida_processes") and package in self._frida_processes:
      self._frida_processes[package].terminate()
      del self._frida_processes[package]
    self._execute_command(
        ['shell', 'am', 'force-stop', package], timeout=timeout)

  def clear_cache(self, package: str, timeout: Optional[float] = None):
    self._execute_command(
        ['shell', 'pm', 'clear', package], timeout=timeout)

  def grant_permissions(self,
                        package: str,
                        permissions: Sequence[str],
                        timeout: Optional[float] = None):
    for permission in permissions:
      logging.info('Granting permission: %r', permission)
      self._execute_command(
          ['shell', 'pm', 'grant', package, permission], timeout=timeout)

  def get_view_hierarchy(self, timeout=None) -> Optional[lxml.etree.Element]:
    #  method `get_view_hierarchy` {{{ # 
    """
    timeout - floating or None

    return lxml.etree.Element or None
    """
    view_hierarchy_output = self._execute_command(["shell", "uiautomator", "dump", "/dev/stdout"], timeout=timeout)
    #logging.info("Getted View Hierarchy Output")
    if view_hierarchy_output:
      view_hierarchy_output = view_hierarchy_output.strip()
      #if view_hierarchy_output.endswith(b"UI hierchary dumped to: /dev/stdout"):
        #view_hierarchy_output = view_hierarchy_output[:-35]
      rindex = view_hierarchy_output.rfind(b">")
      view_hierarchy_output = view_hierarchy_output[:rindex+1] if rindex!=-1\
          else view_hierarchy_output
      #logging.info("Fetched View Hierarchy XML: {:}".format(view_hierarchy_output.decode("utf-8")))
      logging.info("Fetched View Hierarchy XML")
      try:
        root = lxml.etree.fromstring(view_hierarchy_output)
        #logging.info("Parsed View Hierarchy XML")
      except lxml.etree.XMLSyntaxError:
        logging.error("View Hierarchy XML Parsing Error!")
        root = None
      return root
    return None
    #  }}} method `get_view_hierarchy` # 

  def get_activity_dumpsys(self,
                           package_name: str,
                           timeout: Optional[float] = None) -> Optional[str]:
    """Returns the activity's dumpsys output in a UTF-8 string."""
    dumpsys_activity_output = self._execute_command(
        ['shell', 'dumpsys', 'activity', package_name, package_name],
        timeout=timeout)
    if dumpsys_activity_output:
      return dumpsys_activity_output.decode('utf-8')

  def get_current_activity(self,
                           timeout: Optional[float] = None) -> Optional[str]:
    """Returns the full activity name that is currently opened to the user.

    The format of the output is `package/package.ActivityName', for example:
    "com.example.vokram/com.example.vokram.MainActivity"

    Args:
      timeout: A timeout to use for this operation. If not set the default
        timeout set on the constructor will be used.

    Returns:
      None if no current activity can be extracted.
    """
    visible_task = self._execute_command(
        ['shell', 'am', 'stack', 'list', '|', 'grep', '-E', 'visible=true'],
        timeout=timeout)

    if not visible_task:
      am_stack_list = self._execute_command(
          ['shell', 'am', 'stack', 'list'],
          timeout=timeout)
      logging.error('Empty visible_task. `am stack list`: %r', am_stack_list)
      return None

    visible_task = visible_task.decode('utf-8')
    if self._platform_sys == 'win32':
      visible_task_list = re.findall(
          r'visible=true topActivity=ComponentInfo{(.+?)}', visible_task)
      if not visible_task_list:
        visible_task = ''
      else:
        visible_task = 'ComponentInfo{' + visible_task_list[0] + '}'

    p = re.compile(r'.*\{(.*)\}')
    matches = p.search(visible_task)
    if matches is None:
      logging.error(
          'Could not extract current activity. Will return nothing. '
          '`am stack list`: %r',
          self._execute_command(['shell', 'am', 'stack', 'list'],
                                timeout=timeout))
      return None

    return matches.group(1)

  def start_screen_pinning(self,
                           full_activity: str,
                           timeout: Optional[float] = None):
    current_task_id = self._fetch_current_task_id(full_activity, timeout)
    if current_task_id == -1:
      logging.info('Could not find task ID for activity [%r]', full_activity)
      raise errors.AdbControllerScreenPinningError
    self._execute_command(
        ['shell', 'am', 'task', 'lock', str(current_task_id)], timeout=timeout)

  def _fetch_current_task_id(self,
                             full_activity_name: str,
                             timeout: Optional[float] = None) -> int:
    """Returns the task ID of the given `full_activity_name`."""
    stack = self._execute_command(
        ['shell', 'am', 'stack', 'list'], timeout=timeout)
    stack_utf8 = stack.decode('utf-8')
    lines = stack_utf8.splitlines()

    if self._platform_sys == 'win32':
      regex = re.compile(
          r'^\ *taskId=(?P<id>[0-9]*): .* visible=true .*{%s}.*' %
          full_activity_name)
    else:
      regex = re.compile(
          r'^\ *taskId=(?P<id>[0-9]*): %s.*visible=true.*$' %
          full_activity_name)

    matches = [regex.search(line) for line in lines]
    for match in matches:
      if match is None:
        continue
      current_task_id_str = match.group('id')
      try:
        current_task_id = int(current_task_id_str)
        return current_task_id
      except ValueError:
        logging.info('Failed to parse task ID [%r].', current_task_id_str)
    logging.error('Could not find current activity in stack list: %r',
                  stack_utf8)
    # At this point if we could not find a task ID, there's nothing we can do.
    return -1

  def get_screen_dimensions(self,
                            timeout: Optional[float] = None) -> Tuple[int, int]:
    """Returns a (height, width)-tuple representing a screen size in pixels."""
    logging.info('Fetching screen dimensions...')
    self._wait_for_device(timeout=timeout)
    adb_output = self._execute_command(['shell', 'wm', 'size'], timeout=timeout)
    assert adb_output, 'Empty response from ADB for screen size.'
    adb_output = adb_output.decode('utf-8')
    # adb_output should be of the form "Physical size: 320x480".
    physical = re.match(r'.*Physical\ssize:\s([0-9]+x[0-9]+).*', adb_output)
    override = re.match(r'.*\nOverride\ssize:\s([0-9]+x[0-9]+).*', adb_output)
    dims_match = override or physical
    assert dims_match, f'Failed to match the screen dimensions. {adb_output}'
    dims = dims_match.group(1)
    logging.info('width x height: %s', dims)
    width, height = tuple(map(int, dims.split('x')))  # Split between W & H
    logging.info('Done fetching screen dimensions: (H x W) = (%r, %r)',
                 height, width)
    return (height, width)

  def get_orientation(self, timeout: Optional[float] = None) -> Optional[str]:
    """Returns the device orientation."""
    logging.info('Getting orientation...')
    dumpsys = self._execute_command(
        ['shell', 'dumpsys', 'input'], timeout=timeout)
    #logging.info('dumpsys: %r', dumpsys)
    if not dumpsys:
      logging.error('Empty dumpsys.')
      return None
    dumpsys = dumpsys.decode('utf-8')
    lines = dumpsys.split('\n')  # Split by lines.
    skip_next = False
    for line in lines:
      # There may be multiple devices in dumpsys. An invalid device can be
      # identified by negative PhysicalWidth.
      physical_width = re.match(r'\s+PhysicalWidth:\s+(-?\d+)px', line)
      if physical_width:
        skip_next = int(physical_width.group(1)) < 0

      surface_orientation = re.match(r'\s+SurfaceOrientation:\s+(\d)', line)
      if surface_orientation is not None:
        if skip_next:
          continue
        orientation = surface_orientation.group(1)
        logging.info('Done getting orientation: %r', orientation)
        return orientation

    logging.error('Could not get the orientation. Returning None.')
    return None

  def rotate_device(self,
                    orientation: adb_pb2.AdbCall.Rotate.Orientation,
                    timeout: Optional[float] = None) -> None:
    """Sets the device to the given `orientation`."""
    self._execute_command(
        ['shell', 'settings', 'put', 'system', 'user_rotation',
         str(orientation)], timeout=timeout)

  def set_touch_indicators(self,
                           show_touches: bool = True,
                           pointer_location: bool = True,
                           timeout: Optional[float] = None) -> None:
    """Sends command to turn touch indicators on/off."""
    logging.info('Setting show_touches indicator to %r', show_touches)
    logging.info('Setting pointer_location indicator to %r', pointer_location)
    show_touches = 1 if show_touches else 0
    pointer_location = 1 if pointer_location else 0
    self._wait_for_device(timeout=timeout)
    self._execute_command([
        'shell', 'settings', 'put', 'system', 'show_touches',
        str(show_touches)], timeout=timeout)
    self._execute_command([
        'shell', 'settings', 'put', 'system', 'pointer_location',
        str(pointer_location)], timeout=timeout)

  def set_bar_visibility(self,
                         navigation: bool = False,
                         status: bool = False,
                         timeout: Optional[float] = None) -> Optional[bytes]:
    """Show or hide navigation and status bars."""
    command = ['shell', 'settings', 'put', 'global', 'policy_control']
    if status and navigation:  # Show both bars.
      command += ['null*']
    elif not status and navigation:  # Hide status(top) bar.
      command += ['immersive.status=*']
    elif status and not navigation:  # Hide navigation(bottom) bar.
      command += ['immersive.navigation=*']
    else:  # Hide both bars.
      command += ['immersive.full=*']

    return self._execute_command(command, timeout=timeout)

  def disable_animations(self, timeout: Optional[float] = None):
    self._execute_command(
        ['shell', 'settings put global window_animation_scale 0.0'],
        timeout=timeout)
    self._execute_command(
        ['shell', 'settings put global transition_animation_scale 0.0'],
        timeout=timeout)
    self._execute_command(
        ['shell', 'settings put global animator_duration_scale 0.0'],
        timeout=timeout)

  def input_tap(self, x: int, y: int, timeout: Optional[float] = None) -> None:
    self._execute_command(
        ['shell', 'input', 'tap', str(x), str(y)], timeout=timeout)

  def input_text(self,
                 input_text: str,
                 timeout: Optional[float] = None) -> Optional[bytes]:
    return self._execute_command(
        ['shell', 'input', 'text', input_text], timeout=timeout)

  def input_key(self,
                key_code: str,
                timeout: Optional[float] = None) -> Optional[bytes]:
    """Presses a keyboard key.

    Please see https://developer.android.com/reference/android/view/KeyEvent for
    values of `key_code`.

    We currently only accept:

    KEYCODE_HOME (constant 3)
    KEYCODE_BACK (constant 4)
    KEYCODE_ENTER (constant 66)

    Args:
      key_code: The keyboard key to press.
      timeout: Optional time limit in seconds.

    Returns:
      The output of running such command as a string, None if it fails.
    """

    accepted_key_codes = { 'KEYCODE_HOME'
                         , 'KEYCODE_BACK'
                         , 'KEYCODE_ENTER' # '\n', '\r'
                         , "KEYCODE_DEL" # '\b'
                         , "KEYCODE_ESCAPE" # '\e'
                         , "KEYCODE_TAB" # '\t'
                         , "KEYCODE_GRAVE" # '`'
                         }
    assert key_code in accepted_key_codes, ('Rejected keycode: %r' % key_code)

    return self._execute_command(['shell', 'input', 'keyevent', key_code],
                                 timeout=timeout)



# Copyright 2023 SJTU X-Lance Lab
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# 
# Revised by Danyang Zhang @X-Lance based on
# 
# Copyright 2021 DeepMind Technologies Limited.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Simple package definition for using with `pip`."""

from distutils import cmd
import os

import pkg_resources
from setuptools import find_packages
from setuptools import setup
from setuptools.command.build_ext import build_ext
from setuptools.command.build_py import build_py

description = """Mobile-Env
Read the README at https://github.com/X-LANCE/Mobile-Env for more information.
"""

_ROOT_DIR = os.path.dirname(os.path.abspath(__file__))

# Tuple of proto message definitions to build Python bindings for. Paths must
# be relative to root directory.
_ANDROID_ENV_PROTOS = (
    'android_env/proto/emulator_controller.proto',
    'android_env/proto/raw_observation.proto',
    'android_env/proto/adb.proto',
    'android_env/proto/task.proto',
)

testing_requirements = [
    'attrs==20.3.0',  # temporary pin to fix pytype issue.
    'pillow',
    'pytype',
    'pytest-xdist',
]


class _GenerateProtoFiles(cmd.Command):
  """Command to generate protobuf bindings for AndroidEnv protos."""

  descriptions = 'Generates Python protobuf bindings for Mobile-Env protos.'
  user_options = []

  def initialize_options(self):
    pass

  def finalize_options(self):
    pass

  def run(self):
    # Import grpc_tools here, after setuptools has installed setup_requires
    # dependencies.
    from grpc_tools import protoc  # pylint: disable=g-import-not-at-top

    grpc_protos_include = pkg_resources.resource_filename(
        'grpc_tools', '_proto')

    for proto_path in _ANDROID_ENV_PROTOS:
      proto_args = [
          'grpc_tools.protoc',
          '--proto_path={}'.format(grpc_protos_include),
          '--proto_path={}'.format(_ROOT_DIR),
          '--python_out={}'.format(_ROOT_DIR),
          '--grpc_python_out={}'.format(_ROOT_DIR),
          os.path.join(_ROOT_DIR, proto_path),
      ]
      if protoc.main(proto_args) != 0:
        raise RuntimeError('ERROR: {}'.format(proto_args))


class _BuildExt(build_ext):
  """Generate protobuf bindings in build_ext stage."""

  def run(self):
    self.run_command('generate_protos')
    build_ext.run(self)


class _BuildPy(build_py):
  """Generate protobuf bindings in build_py stage."""

  def run(self):
    self.run_command('generate_protos')
    build_py.run(self)

setup(
    name='mobile-env-rl',
    version='2.0',
    description='Mobile-Env: A Universal Platform for Training and Evaluation of Mobile Interaction',
    long_description=description,
    author='Danyang Zhang @X-Lance',
    license='Apache License, Version 2.0',
    keywords='InfoUI interaction',
    url='https://github.com/X-LANCE/Mobile-Env',
    packages=find_packages(exclude=['examples']),
    package_data={"": ["proto/*.proto"]},
    setup_requires=[
        'grpcio-tools',
    ],
    install_requires=[
        'absl-py>=0.1.0',
        'dm_env',
        'grpcio',
        'mock',
        'numpy',
        'pexpect>=4.8.0',
        'portpicker>=1.2.0',
        'protobuf>=2.6',
        'lxml',
        'torch',
        'torchvision',
        'transformers'
    ],
    extras_require={
        #'acme': ['dm-acme'],
        'easyocr': ['easyocr'],
        'pygame': ['pygame'],
        'gym': ['gym'],
        'testing': testing_requirements,
    },
    cmdclass={
        'build_ext': _BuildExt,
        'build_py': _BuildPy,
        'generate_protos': _GenerateProtoFiles,
    },
)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: android_env/proto/adb.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1b\x61ndroid_env/proto/adb.proto\x12\x0b\x61ndroid_env\"\x85\x10\n\x07\x41\x64\x62\x43\x61ll\x12\x36\n\x0binstall_apk\x18\x01 \x01(\x0b\x32\x1f.android_env.AdbCall.InstallApkH\x00\x12<\n\x0estart_activity\x18\x02 \x01(\x0b\x32\".android_env.AdbCall.StartActivityH\x00\x12\x34\n\nforce_stop\x18\x03 \x01(\x0b\x32\x1e.android_env.AdbCall.ForceStopH\x00\x12\x36\n\x0b\x63lear_cache\x18\x04 \x01(\x0b\x32\x1f.android_env.AdbCall.ClearCacheH\x00\x12\x42\n\x11grant_permissions\x18\x05 \x01(\x0b\x32%.android_env.AdbCall.GrantPermissionsH\x00\x12\'\n\x03tap\x18\x06 \x01(\x0b\x32\x18.android_env.AdbCall.TapH\x00\x12\x38\n\x0cpress_button\x18\x07 \x01(\x0b\x32 .android_env.AdbCall.PressButtonH\x00\x12-\n\x06rotate\x18\x08 \x01(\x0b\x32\x1b.android_env.AdbCall.RotateH\x00\x12U\n\x1bstart_accessibility_service\x18\t \x01(\x0b\x32..android_env.AdbCall.StartAccessibilityServiceH\x00\x12G\n\x14start_screen_pinning\x18\n \x01(\x0b\x32\'.android_env.AdbCall.StartScreenPinningH\x00\x12I\n\x15start_random_activity\x18\x0b \x01(\x0b\x32(.android_env.AdbCall.StartRandomActivityH\x00\x12R\n\x1a\x66orce_stop_random_activity\x18\x0c \x01(\x0b\x32,.android_env.AdbCall.ForceStopRandomActivityH\x00\x12T\n\x1b\x63lear_cache_random_activity\x18\r \x01(\x0b\x32-.android_env.AdbCall.ClearCacheRandomActivityH\x00\x12\x44\n\x12\x64isable_animations\x18\x0e \x01(\x0b\x32&.android_env.AdbCall.DisableAnimationsH\x00\x12\x38\n\x0cstart_intent\x18\x0f \x01(\x0b\x32 .android_env.AdbCall.StartIntentH\x00\x12\x42\n\x11uninstall_package\x18\x10 \x01(\x0b\x32%.android_env.AdbCall.UninstallPackageH\x00\x1av\n\nInstallApk\x12@\n\nfilesystem\x18\x02 \x01(\x0b\x32*.android_env.AdbCall.InstallApk.FilesystemH\x00\x1a\x1a\n\nFilesystem\x12\x0c\n\x04path\x18\x01 \x01(\tB\n\n\x08location\x1aH\n\rStartActivity\x12\x15\n\rfull_activity\x18\x01 \x01(\t\x12\x12\n\nextra_args\x18\x02 \x03(\t\x12\x0c\n\x04\x66lag\x18\x03 \x01(\t\x1a\x45\n\x0bStartIntent\x12\x10\n\x08\x64\x61ta_uri\x18\x01 \x01(\t\x12\x0e\n\x06\x61\x63tion\x18\x02 \x01(\t\x12\x14\n\x0cpackage_name\x18\x03 \x01(\t\x1aU\n\x13StartRandomActivity\x12\x15\n\ractivity_list\x18\x01 \x03(\t\x12\x12\n\nextra_args\x18\x02 \x03(\t\x12\x13\n\x0btimeout_sec\x18\x04 \x01(\x02\x1a(\n\x10UninstallPackage\x12\x14\n\x0cpackage_name\x18\x01 \x01(\t\x1a!\n\tForceStop\x12\x14\n\x0cpackage_name\x18\x01 \x01(\t\x1a\x19\n\x17\x46orceStopRandomActivity\x1a\x13\n\x11\x44isableAnimations\x1a\"\n\nClearCache\x12\x14\n\x0cpackage_name\x18\x01 \x01(\t\x1a\x1a\n\x18\x43learCacheRandomActivity\x1a=\n\x10GrantPermissions\x12\x14\n\x0cpackage_name\x18\x01 \x01(\t\x12\x13\n\x0bpermissions\x18\x02 \x03(\t\x1a\x1b\n\x03Tap\x12\t\n\x01x\x18\x01 \x01(\x05\x12\t\n\x01y\x18\x02 \x01(\x05\x1a\x64\n\x0bPressButton\x12\x37\n\x06\x62utton\x18\x01 \x01(\x0e\x32\'.android_env.AdbCall.PressButton.Button\"\x1c\n\x06\x42utton\x12\x08\n\x04HOME\x10\x00\x12\x08\n\x04\x42\x41\x43K\x10\x01\x1a\x9c\x01\n\x06Rotate\x12<\n\x0borientation\x18\x01 \x01(\x0e\x32\'.android_env.AdbCall.Rotate.Orientation\"T\n\x0bOrientation\x12\x0e\n\nPORTRAIT_0\x10\x00\x12\x10\n\x0cLANDSCAPE_90\x10\x01\x12\x10\n\x0cPORTRAIT_180\x10\x02\x12\x11\n\rLANDSCAPE_270\x10\x03\x1a\x31\n\x19StartAccessibilityService\x12\x14\n\x0c\x66ull_service\x18\x01 \x01(\t\x1a+\n\x12StartScreenPinning\x12\x15\n\rfull_activity\x18\x01 \x01(\tB\t\n\x07\x63ommandb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'android_env.proto.adb_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _ADBCALL._serialized_start=45
  _ADBCALL._serialized_end=2098
  _ADBCALL_INSTALLAPK._serialized_start=1099
  _ADBCALL_INSTALLAPK._serialized_end=1217
  _ADBCALL_INSTALLAPK_FILESYSTEM._serialized_start=1179
  _ADBCALL_INSTALLAPK_FILESYSTEM._serialized_end=1205
  _ADBCALL_STARTACTIVITY._serialized_start=1219
  _ADBCALL_STARTACTIVITY._serialized_end=1291
  _ADBCALL_STARTINTENT._serialized_start=1293
  _ADBCALL_STARTINTENT._serialized_end=1362
  _ADBCALL_STARTRANDOMACTIVITY._serialized_start=1364
  _ADBCALL_STARTRANDOMACTIVITY._serialized_end=1449
  _ADBCALL_UNINSTALLPACKAGE._serialized_start=1451
  _ADBCALL_UNINSTALLPACKAGE._serialized_end=1491
  _ADBCALL_FORCESTOP._serialized_start=1493
  _ADBCALL_FORCESTOP._serialized_end=1526
  _ADBCALL_FORCESTOPRANDOMACTIVITY._serialized_start=1528
  _ADBCALL_FORCESTOPRANDOMACTIVITY._serialized_end=1553
  _ADBCALL_DISABLEANIMATIONS._serialized_start=1555
  _ADBCALL_DISABLEANIMATIONS._serialized_end=1574
  _ADBCALL_CLEARCACHE._serialized_start=1576
  _ADBCALL_CLEARCACHE._serialized_end=1610
  _ADBCALL_CLEARCACHERANDOMACTIVITY._serialized_start=1612
  _ADBCALL_CLEARCACHERANDOMACTIVITY._serialized_end=1638
  _ADBCALL_GRANTPERMISSIONS._serialized_start=1640
  _ADBCALL_GRANTPERMISSIONS._serialized_end=1701
  _ADBCALL_TAP._serialized_start=1703
  _ADBCALL_TAP._serialized_end=1730
  _ADBCALL_PRESSBUTTON._serialized_start=1732
  _ADBCALL_PRESSBUTTON._serialized_end=1832
  _ADBCALL_PRESSBUTTON_BUTTON._serialized_start=1804
  _ADBCALL_PRESSBUTTON_BUTTON._serialized_end=1832
  _ADBCALL_ROTATE._serialized_start=1835
  _ADBCALL_ROTATE._serialized_end=1991
  _ADBCALL_ROTATE_ORIENTATION._serialized_start=1907
  _ADBCALL_ROTATE_ORIENTATION._serialized_end=1991
  _ADBCALL_STARTACCESSIBILITYSERVICE._serialized_start=1993
  _ADBCALL_STARTACCESSIBILITYSERVICE._serialized_end=2042
  _ADBCALL_STARTSCREENPINNING._serialized_start=2044
  _ADBCALL_STARTSCREENPINNING._serialized_end=2087
# @@protoc_insertion_point(module_scope)



# coding=utf-8
# Copyright 2021 DeepMind Technologies Limited.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Fake Simulator for testing AndroidEnv infrastructure."""

import random
import threading
import time
from typing import Dict, List, Optional, Tuple

from absl import logging
from android_env.components import adb_controller
from android_env.components import log_stream
from android_env.components.simulators import base_simulator
import numpy as np


class FakeStream():
  """This class simulates the logs coming from ADB."""

  def __init__(self):
    self._values = [
        '',
        self._make_stdout('reward: 0.5'),
        self._make_stdout('reward: 1.0'),
        self._make_stdout('extra: my_extra: [1.0]'),
        self._make_stdout('episode end'),
    ]
    self._kill = False
    self._lock = threading.Lock()

  def _make_stdout(self, data):
    """Returns a valid log output with given data as message."""
    return f'         1553110400.424  5583  5658 D Tag: {data}'

  def kill(self):
    self._kill = True

  def __iter__(self):
    while True:
      if self._kill:
        return
      else:
        with self._lock:
          next_value = random.choices(
              self._values, weights=[0.49, 0.15, 0.15, 0.15, 0.01], k=1)[0]
          time.sleep(0.1)
        yield next_value


class FakeLogStream(log_stream.LogStream):
  """FakeLogStream class that wraps a FakeStream."""

  def __init__(self):
    self.stream = FakeStream()
    self.stream_is_alive = True
    self._verbose = False

  def _get_stream_output(self):
    return self.stream

  def stop_stream(self):
    self.stream_is_alive = False
    self.stream.kill()


class FakeAdbController(adb_controller.AdbController):
  """Fake adb controller for FakeSimulator."""

  def __init__(self,
               screen_dimensions: Tuple[int, int],
               fake_activity: str,
               *args, **kwargs):
    super().__init__(*args, **kwargs)
    self._screen_dimensions = screen_dimensions
    self._fake_activity = fake_activity

  def _execute_command(
      self,
      args: List[str],
      timeout: Optional[float] = None) -> Optional[bytes]:
    del args, timeout
    return bytes('fake output', 'utf-8')

  def _wait_for_device(
      self,
      max_tries: int = 20,
      sleep_time: float = 1.0,
      timeout: Optional[float] = None) -> None:
    """Fake adb controller does not have to wait for a device."""
    pass

  def get_screen_dimensions(
      self, timeout: Optional[float] = None) -> Tuple[int, int]:
    return self._screen_dimensions

  def get_current_activity(
      self, timeout: Optional[float] = None) -> Optional[str]:
    return self._fake_activity

  def is_package_installed(
      self, package_name: str, timeout: Optional[float] = None) -> bool:
    return True

  def start_screen_pinning(
      self, full_activity: str, timeout: Optional[float] = None):
    pass


class FakeSimulator(base_simulator.BaseSimulator):
  """FakeSimulator class."""

  def __init__(self,
               screen_dimensions: Tuple[int, int] = (480, 320),
               fake_activity: str = '',
               **kwargs):
    """FakeSimulator class that can replace EmulatorSimulator in AndroidEnv.

    Args:
      screen_dimensions: desired screen dimensions in pixels.
      fake_activity: if FakeSimulator is used in combination with a
        task_pb2.Task() that has an expected_activity, we can mock an
        activity here. If this activity does not match the expected_activity,
        an error will be raised.
      **kwargs: other keyword arguments for the base class.
    """
    super().__init__(**kwargs)
    self._screen_dimensions = screen_dimensions
    self._fake_activity = fake_activity
    self._adb_controller = self.create_adb_controller()
    logging.info('Created FakeSimulator.')

  def adb_device_name(self) -> str:
    return 'fake_simulator'

  def create_adb_controller(self):
    return FakeAdbController(
        screen_dimensions=self._screen_dimensions,
        fake_activity=self._fake_activity)

  def _restart_impl(self) -> None:
    pass

  def _launch_impl(self) -> None:
    pass

  def send_action(self, action: Dict[str, np.ndarray]) -> None:
    del action

  def _get_observation(self) -> Optional[List[np.ndarray]]:
    image = np.zeros(shape=(*self._screen_dimensions, 3), dtype=np.uint8)
    timestamp = np.random.randint(100, dtype=np.int64)
    return [image, timestamp]

  def _create_log_stream(self) -> log_stream.LogStream:
    return FakeLogStream()



# coding=utf-8
# vim: set tabstop=2 shiftwidth=2:
# Copyright 2023 SJTU X-Lance Lab
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# 
# Revised by Danyang Zhang @X-Lance based on
# 
# Copyright 2021 DeepMind Technologies Limited.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""A class that launches a thread to read Android logcat outputs."""

import re
import threading
# `typing.Pattern` has been deprecated in Python 3.9 in favor of `re.Pattern`,
# but it is not available even in slightly older Python versions.
# Please see https://www.python.org/dev/peps/pep-0585/
from typing import Callable, Match, NamedTuple, Optional, Pattern, Iterable

from absl import logging
from android_env.components import log_stream as log_stream_lib
from android_env.components import thread_function
from android_env.proto import task_pb2
from android_env.components import event_listeners


#class EventListener(NamedTuple):
  #regexp: Pattern[str]
  #handler_fn: Callable[[Pattern[str], Match[str]], None]


class LogcatThread(thread_function.ThreadFunction):
  """Reads ADB logcat entries in a separate thread."""

  def __init__(
      self,
      log_stream: log_stream_lib.LogStream,
      #log_parsing_config: task_pb2.LogParsingConfig,
      log_filter: Iterable[str],
      lock: threading.Lock,
      name: str = 'logcat'):
    """Initializes this LogcatThread with optional filters.

    Please see https://developer.android.com/studio/command-line/logcat for more
    info on `logcat`.

    Args:
      log_stream: Stream of logs from simulator.
      #log_parsing_config: Determines the types of messages we want logcat to
        #match. Contains `filters` and `log_regexps`.
      log_filter: Log filters given as an itrable of string.
      lock: threading.Lock
      name: Name of the thread.
    """

    #self._regexps = log_parsing_config.log_regexps # zdy
    #self._listeners = {}
    self._listeners = [] # zdy
    self._lock = lock

    self._desired_event = None
    self._thread_event = threading.Event()

    #self._max_buffer_size = 100 # zdy
    self._log_stream = log_stream
    #self._log_stream.set_log_filters(list(log_parsing_config.filters))
    self._log_stream.set_log_filters(list(log_filter))

    self._stdout = self._log_stream.get_stream_output()

    super().__init__(block_input=True, block_output=False, name=name)

  def add_event_listener(self, event_listener: event_listeners.LogEvent) -> None:
    """Adds `fn` to the list of handlers to call when `event` occurs."""
    #event_regexp = event_listener.regexp
    #if event_regexp not in self._listeners:
      #self._listeners[event_regexp] = []
    #self._listeners[event_regexp].append(event_listener.handler_fn)
    self._listeners.append(event_listener) # zdy

  def remove_event_listener(self, event_listener: event_listeners.LogEvent) -> None:
    """Removes `fn` from the list of handlers to call when `event` occurs."""
    #event_regexp = event_listener.regexp
    #if event_regexp not in self._listeners:
      #logging.error('Event: %r is not registered.', event_regexp)
      #return
    #self._listeners[event_regexp].remove(event_listener.handler_fn)
    try:
      self._listeners.remove(event_listener)
    except ValueError:
      logging.error('Event: %r is not registered.', event_regexp)

  # ZDY_COMMENT: will response to the events happened before the invocation but
  # not being handled yet as well
  def wait(self,
           #event: Optional[Pattern[str]] = None, # zdy
           event: event_listeners.LogEvent = None,
           timeout_sec: Optional[float] = None) -> None:
    """Blocks (caller) execution for up to `timeout_sec` until `event` is fired.

    Args:
      event: Event to wait for. If None, any new event will cause this function
        to return.
      timeout_sec: Maximum time to block waiting for an event.
    """
    self._desired_event = event
    self._thread_event.wait(timeout=timeout_sec)
    self._thread_event.clear()

  def kill(self):
    self._log_stream.stop_stream()

    super().kill()

  def main(self) -> None:
    # pylint: disable=g-line-too-long
    # Format is: "TIME_SEC PID TID PRIORITY TAG: MESSAGE"
    #
    # Example:
    #  '         1553110400.424  5583  5658 D NostalgicRacer: com.google.example.games.nostalgicracer.views.renderers.OpenGLRenderDriver@912fb8.onSurfaceChanged 480x320'    #
    #
    # If a log_prefix is given, then the format becomes:
    # "TIME_SEC PID TID PRIORITY TAG: LOG_PREFIX MESSAGE"
    # pylint: enable=g-line-too-long

    regexp = r"""
      ^                                   # Beginning of the line.
      [ ]+(?P<timestamp>[0-9]+\.[0-9]+)   # Spaces and a float.
      [ ]+(?P<pid>[0-9]+)                 # Spaces and an int.
      [ ]+(?P<tid>[0-9]+)                 # Spaces and an int.
      [ ]+(?P<priority>.)                 # Spaces and any single character.
      [ ]+(?P<tag>[^:]*):                 # Spaces and any char that's not ':'.
    """

    regexp += r"""[ ](?P<message>.*)$"""
    logline_re = re.compile(regexp, re.VERBOSE)

    for line in self._stdout:
      # We never hand back control to ThreadFunction._run() so we need to
      # explicitly check for self._should_run here.
      if not self._should_run:
        break

      if not line:  # Skip empty lines.
        continue

      # We're currently only consuming `message`, but we may use the other
      # fields in the future.
      # ZDY_COMMENT: Actually, I wondered if all the Android comments satisfy
      # the desired format. Maybe it is. I'd better to confirm that. e.g.,
      # export a running log and have a check.
      matches = logline_re.match(line)
      if not matches or len(matches.groups()) != 6:
        continue

      content = matches.group('message')
      with self._lock:
        for evnt_lstn in self._listeners:
          evnt_lstn.set(content)
          if evnt_lstn.is_set():
            if self._desired_event is not None and self._desired_event is evnt_lstn or\
                self._desired_event is None:
              self._thread_event.set()

      #for ev, listeners in self._listeners.items():
        #ev_matches = ev.match(content)
        #if ev_matches:
          ## Unblock consumers that may be waiting for events.
          #if not self._thread_event.is_set():
            #if self._desired_event:
              #if self._desired_event == ev:
                #self._thread_event.set()
            #else:
              #self._thread_event.set()
#
          ## Notify listeners.
          #for listener in listeners:
            #listener(ev, ev_matches)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: android_env/proto/task.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from android_env.proto import adb_pb2 as android__env_dot_proto_dot_adb__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1c\x61ndroid_env/proto/task.proto\x12\x0b\x61ndroid_env\x1a\x1b\x61ndroid_env/proto/adb.proto\":\n\tAppScreen\x12\x10\n\x08\x61\x63tivity\x18\x01 \x01(\t\x12\x1b\n\x13view_hierarchy_path\x18\x02 \x03(\t\"S\n\x10WaitForAppScreen\x12*\n\napp_screen\x18\x01 \x01(\x0b\x32\x16.android_env.AppScreen\x12\x13\n\x0btimeout_sec\x18\x02 \x01(\x02\"9\n\x0c\x43heckInstall\x12\x14\n\x0cpackage_name\x18\x01 \x01(\t\x12\x13\n\x0btimeout_sec\x18\x02 \x01(\x02\"6\n\x0eWaitForMessage\x12\x0f\n\x07message\x18\x01 \x01(\t\x12\x13\n\x0btimeout_sec\x18\x02 \x01(\x02\"\x19\n\x05Sleep\x12\x10\n\x08time_sec\x18\x01 \x01(\x02\"\xdb\x01\n\x10SuccessCondition\x12\x13\n\x0bnum_retries\x18\x01 \x01(\x05\x12<\n\x13wait_for_app_screen\x18\x02 \x01(\x0b\x32\x1d.android_env.WaitForAppScreenH\x00\x12\x32\n\rcheck_install\x18\x03 \x01(\x0b\x32\x19.android_env.CheckInstallH\x00\x12\x37\n\x10wait_for_message\x18\x04 \x01(\x0b\x32\x1b.android_env.WaitForMessageH\x00\x42\x07\n\x05\x63heck\"\x9c\x01\n\tSetupStep\x12\x38\n\x11success_condition\x18\x01 \x01(\x0b\x32\x1d.android_env.SuccessCondition\x12(\n\x08\x61\x64\x62_call\x18\x02 \x01(\x0b\x32\x14.android_env.AdbCallH\x00\x12#\n\x05sleep\x18\x03 \x01(\x0b\x32\x12.android_env.SleepH\x00\x42\x06\n\x04step\"\xb7\x02\n\tArraySpec\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x05shape\x18\x02 \x03(\x05\x12.\n\x05\x64type\x18\x03 \x01(\x0e\x32\x1f.android_env.ArraySpec.DataType\"\xdc\x01\n\x08\x44\x61taType\x12\x15\n\x11INVALID_DATA_TYPE\x10\x00\x12\t\n\x05\x46LOAT\x10\x01\x12\n\n\x06\x44OUBLE\x10\x02\x12\x08\n\x04INT8\x10\x03\x12\t\n\x05INT16\x10\x04\x12\t\n\x05INT32\x10\x05\x12\t\n\x05INT64\x10\x06\x12\t\n\x05UINT8\x10\x07\x12\n\n\x06UINT16\x10\x08\x12\n\n\x06UINT32\x10\t\x12\n\n\x06UINT64\x10\n\x12\x08\n\x04\x42OOL\x10\x0b\x12\r\n\tSTRING_U1\x10\x0c\x12\x0e\n\nSTRING_U16\x10\r\x12\x0e\n\nSTRING_U25\x10\x0e\x12\x0f\n\x0bSTRING_U250\x10\x0f\"\xcc\n\n\x0b\x45ventSource\x12<\n\x0etext_recognize\x18\x03 \x01(\x0b\x32\".android_env.EventSource.TextEventH\x00\x12\x39\n\x0btext_detect\x18\x04 \x01(\x0b\x32\".android_env.EventSource.TextEventH\x00\x12\x41\n\x0eicon_recognize\x18\x05 \x01(\x0b\x32\'.android_env.EventSource.IconRecogEventH\x00\x12>\n\x0bicon_detect\x18\x06 \x01(\x0b\x32\'.android_env.EventSource.IconRecogEventH\x00\x12=\n\nicon_match\x18\x07 \x01(\x0b\x32\'.android_env.EventSource.IconMatchEventH\x00\x12\x44\n\x11icon_detect_match\x18\x08 \x01(\x0b\x32\'.android_env.EventSource.IconMatchEventH\x00\x12K\n\x14view_hierarchy_event\x18\t \x01(\x0b\x32+.android_env.EventSource.ViewHierarchyEventH\x00\x12\x36\n\tlog_event\x18\n \x01(\x0b\x32!.android_env.EventSource.LogEventH\x00\x12\n\n\x02id\x18\x0c \x01(\x05\x12=\n\rrepeatability\x18\x0e \x01(\x0e\x32&.android_env.EventSource.Repeatability\x1a\x36\n\x04Rect\x12\n\n\x02x0\x18\x02 \x01(\x02\x12\n\n\x02y0\x18\x03 \x01(\x02\x12\n\n\x02x1\x18\x04 \x01(\x02\x12\n\n\x02y1\x18\x05 \x01(\x02\x1aH\n\tTextEvent\x12\x0e\n\x06\x65xpect\x18\x01 \x01(\t\x12+\n\x04rect\x18\x02 \x01(\x0b\x32\x1d.android_env.EventSource.Rect\x1aL\n\x0eIconRecogEvent\x12\r\n\x05\x63lass\x18\x01 \x01(\t\x12+\n\x04rect\x18\x02 \x01(\x0b\x32\x1d.android_env.EventSource.Rect\x1aK\n\x0eIconMatchEvent\x12\x0c\n\x04path\x18\x01 \x01(\t\x12+\n\x04rect\x18\x02 \x01(\x0b\x32\x1d.android_env.EventSource.Rect\x1a\xe3\x02\n\x12ViewHierarchyEvent\x12\x1b\n\x13view_hierarchy_path\x18\x01 \x03(\t\x12H\n\nproperties\x18\x02 \x03(\x0b\x32\x34.android_env.EventSource.ViewHierarchyEvent.Property\x1a\xad\x01\n\x08Property\x12\x15\n\rproperty_name\x18\x01 \x01(\t\x12>\n\x04sign\x18\x02 \x01(\x0e\x32\x30.android_env.EventSource.ViewHierarchyEvent.Sign\x12\x11\n\x07pattern\x18\x03 \x01(\tH\x00\x12\x11\n\x07integer\x18\x04 \x01(\x05H\x00\x12\x12\n\x08\x66loating\x18\x05 \x01(\x02H\x00\x42\x10\n\x0eproperty_value\"6\n\x04Sign\x12\x06\n\x02\x45Q\x10\x00\x12\x06\n\x02LE\x10\x01\x12\x06\n\x02LT\x10\x02\x12\x06\n\x02GE\x10\x03\x12\x06\n\x02GT\x10\x04\x12\x06\n\x02NE\x10\x05\x1a,\n\x08LogEvent\x12\x0f\n\x07\x66ilters\x18\x01 \x03(\t\x12\x0f\n\x07pattern\x18\x02 \x01(\t\"2\n\rRepeatability\x12\x08\n\x04NONE\x10\x00\x12\x08\n\x04LAST\x10\x01\x12\r\n\tUNLIMITED\x10\x02\x42\x07\n\x05\x65vent\"\x97\x02\n\tEventSlot\x12)\n\x04type\x18\x01 \x01(\x0e\x32\x1b.android_env.EventSlot.Type\x12\n\n\x02id\x18\x02 \x01(\x05\x12\x31\n\x06\x65vents\x18\x03 \x03(\x0b\x32!.android_env.EventSlot.FakeSource\x12\x14\n\x0cprerequisite\x18\x04 \x03(\x05\x12\x16\n\x0etransformation\x18\x05 \x03(\t\x1aM\n\nFakeSource\x12\x0c\n\x02id\x18\x01 \x01(\x05H\x00\x12\'\n\x05\x65vent\x18\x02 \x01(\x0b\x32\x16.android_env.EventSlotH\x00\x42\x08\n\x06source\"#\n\x04Type\x12\n\n\x06SINGLE\x10\x00\x12\x07\n\x03\x41ND\x10\x01\x12\x06\n\x02OR\x10\x02\"\xc2\x02\n\x0eTaskEventSlots\x12.\n\x0escore_listener\x18\x01 \x01(\x0b\x32\x16.android_env.EventSlot\x12/\n\x0freward_listener\x18\x02 \x01(\x0b\x32\x16.android_env.EventSlot\x12\x34\n\x14\x65pisode_end_listener\x18\x03 \x01(\x0b\x32\x16.android_env.EventSlot\x12.\n\x0e\x65xtra_listener\x18\x04 \x01(\x0b\x32\x16.android_env.EventSlot\x12\x33\n\x13json_extra_listener\x18\x05 \x01(\x0b\x32\x16.android_env.EventSlot\x12\x34\n\x14instruction_listener\x18\x06 \x01(\x0b\x32\x16.android_env.EventSlot\"\xaa\x03\n\x04Task\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x03 \x01(\t\x12+\n\x0bsetup_steps\x18\x04 \x03(\x0b\x32\x16.android_env.SetupStep\x12+\n\x0breset_steps\x18\x05 \x03(\x0b\x32\x16.android_env.SetupStep\x12\x33\n\x13\x65xpected_app_screen\x18\x06 \x01(\x0b\x32\x16.android_env.AppScreen\x12\x18\n\x10max_duration_sec\x18\x07 \x01(\x02\x12\x15\n\rmax_num_steps\x18\x08 \x01(\x05\x12/\n\revent_sources\x18\t \x03(\x0b\x32\x18.android_env.EventSource\x12\x30\n\x0b\x65vent_slots\x18\n \x01(\x0b\x32\x1b.android_env.TaskEventSlots\x12+\n\x0b\x65xtras_spec\x18\x0b \x03(\x0b\x32\x16.android_env.ArraySpec\x12\x0f\n\x07\x63ommand\x18\x0c \x03(\t\x12\x12\n\nvocabulary\x18\r \x03(\tb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'android_env.proto.task_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _APPSCREEN._serialized_start=74
  _APPSCREEN._serialized_end=132
  _WAITFORAPPSCREEN._serialized_start=134
  _WAITFORAPPSCREEN._serialized_end=217
  _CHECKINSTALL._serialized_start=219
  _CHECKINSTALL._serialized_end=276
  _WAITFORMESSAGE._serialized_start=278
  _WAITFORMESSAGE._serialized_end=332
  _SLEEP._serialized_start=334
  _SLEEP._serialized_end=359
  _SUCCESSCONDITION._serialized_start=362
  _SUCCESSCONDITION._serialized_end=581
  _SETUPSTEP._serialized_start=584
  _SETUPSTEP._serialized_end=740
  _ARRAYSPEC._serialized_start=743
  _ARRAYSPEC._serialized_end=1054
  _ARRAYSPEC_DATATYPE._serialized_start=834
  _ARRAYSPEC_DATATYPE._serialized_end=1054
  _EVENTSOURCE._serialized_start=1057
  _EVENTSOURCE._serialized_end=2413
  _EVENTSOURCE_RECT._serialized_start=1665
  _EVENTSOURCE_RECT._serialized_end=1719
  _EVENTSOURCE_TEXTEVENT._serialized_start=1721
  _EVENTSOURCE_TEXTEVENT._serialized_end=1793
  _EVENTSOURCE_ICONRECOGEVENT._serialized_start=1795
  _EVENTSOURCE_ICONRECOGEVENT._serialized_end=1871
  _EVENTSOURCE_ICONMATCHEVENT._serialized_start=1873
  _EVENTSOURCE_ICONMATCHEVENT._serialized_end=1948
  _EVENTSOURCE_VIEWHIERARCHYEVENT._serialized_start=1951
  _EVENTSOURCE_VIEWHIERARCHYEVENT._serialized_end=2306
  _EVENTSOURCE_VIEWHIERARCHYEVENT_PROPERTY._serialized_start=2077
  _EVENTSOURCE_VIEWHIERARCHYEVENT_PROPERTY._serialized_end=2250
  _EVENTSOURCE_VIEWHIERARCHYEVENT_SIGN._serialized_start=2252
  _EVENTSOURCE_VIEWHIERARCHYEVENT_SIGN._serialized_end=2306
  _EVENTSOURCE_LOGEVENT._serialized_start=2308
  _EVENTSOURCE_LOGEVENT._serialized_end=2352
  _EVENTSOURCE_REPEATABILITY._serialized_start=2354
  _EVENTSOURCE_REPEATABILITY._serialized_end=2404
  _EVENTSLOT._serialized_start=2416
  _EVENTSLOT._serialized_end=2695
  _EVENTSLOT_FAKESOURCE._serialized_start=2581
  _EVENTSLOT_FAKESOURCE._serialized_end=2658
  _EVENTSLOT_TYPE._serialized_start=2660
  _EVENTSLOT_TYPE._serialized_end=2695
  _TASKEVENTSLOTS._serialized_start=2698
  _TASKEVENTSLOTS._serialized_end=3020
  _TASK._serialized_start=3023
  _TASK._serialized_end=3449
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: android_env/proto/emulator_controller.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n+android_env/proto/emulator_controller.proto\x12\x19\x61ndroid.emulation.control\x1a\x1bgoogle/protobuf/empty.proto\"\xd7\x01\n\nVmRunState\x12=\n\x05state\x18\x01 \x01(\x0e\x32..android.emulation.control.VmRunState.RunState\"\x89\x01\n\x08RunState\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x0b\n\x07RUNNING\x10\x01\x12\x0e\n\nRESTORE_VM\x10\x02\x12\n\n\x06PAUSED\x10\x03\x12\x0b\n\x07SAVE_VM\x10\x04\x12\x0c\n\x08SHUTDOWN\x10\x05\x12\r\n\tTERMINATE\x10\x07\x12\t\n\x05RESET\x10\t\x12\x12\n\x0eINTERNAL_ERROR\x10\n\"\"\n\x0eParameterValue\x12\x10\n\x04\x64\x61ta\x18\x01 \x03(\x02\x42\x02\x10\x01\"\xc3\x04\n\x12PhysicalModelValue\x12J\n\x06target\x18\x01 \x01(\x0e\x32:.android.emulation.control.PhysicalModelValue.PhysicalType\x12\x43\n\x06status\x18\x02 \x01(\x0e\x32\x33.android.emulation.control.PhysicalModelValue.State\x12\x38\n\x05value\x18\x03 \x01(\x0b\x32).android.emulation.control.ParameterValue\"U\n\x05State\x12\x06\n\x02OK\x10\x00\x12\x17\n\nNO_SERVICE\x10\xfd\xff\xff\xff\xff\xff\xff\xff\xff\x01\x12\x15\n\x08\x44ISABLED\x10\xfe\xff\xff\xff\xff\xff\xff\xff\xff\x01\x12\x14\n\x07UNKNOWN\x10\xff\xff\xff\xff\xff\xff\xff\xff\xff\x01\"\x8a\x02\n\x0cPhysicalType\x12\x0c\n\x08POSITION\x10\x00\x12\x0c\n\x08ROTATION\x10\x01\x12\x12\n\x0eMAGNETIC_FIELD\x10\x02\x12\x0f\n\x0bTEMPERATURE\x10\x03\x12\r\n\tPROXIMITY\x10\x04\x12\t\n\x05LIGHT\x10\x05\x12\x0c\n\x08PRESSURE\x10\x06\x12\x0c\n\x08HUMIDITY\x10\x07\x12\x0c\n\x08VELOCITY\x10\x08\x12\x12\n\x0e\x41MBIENT_MOTION\x10\t\x12\x10\n\x0cHINGE_ANGLE0\x10\n\x12\x10\n\x0cHINGE_ANGLE1\x10\x0b\x12\x10\n\x0cHINGE_ANGLE2\x10\x0c\x12\r\n\tROLLABLE0\x10\r\x12\r\n\tROLLABLE1\x10\x0e\x12\r\n\tROLLABLE2\x10\x0f\"\xf8\x03\n\x0bSensorValue\x12\x41\n\x06target\x18\x01 \x01(\x0e\x32\x31.android.emulation.control.SensorValue.SensorType\x12<\n\x06status\x18\x02 \x01(\x0e\x32,.android.emulation.control.SensorValue.State\x12\x38\n\x05value\x18\x03 \x01(\x0b\x32).android.emulation.control.ParameterValue\"U\n\x05State\x12\x06\n\x02OK\x10\x00\x12\x17\n\nNO_SERVICE\x10\xfd\xff\xff\xff\xff\xff\xff\xff\xff\x01\x12\x15\n\x08\x44ISABLED\x10\xfe\xff\xff\xff\xff\xff\xff\xff\xff\x01\x12\x14\n\x07UNKNOWN\x10\xff\xff\xff\xff\xff\xff\xff\xff\xff\x01\"\xd6\x01\n\nSensorType\x12\x10\n\x0c\x41\x43\x43\x45LERATION\x10\x00\x12\r\n\tGYROSCOPE\x10\x01\x12\x12\n\x0eMAGNETIC_FIELD\x10\x02\x12\x0f\n\x0bORIENTATION\x10\x03\x12\x0f\n\x0bTEMPERATURE\x10\x04\x12\r\n\tPROXIMITY\x10\x05\x12\t\n\x05LIGHT\x10\x06\x12\x0c\n\x08PRESSURE\x10\x07\x12\x0c\n\x08HUMIDITY\x10\x08\x12\x1f\n\x1bMAGNETIC_FIELD_UNCALIBRATED\x10\t\x12\x1a\n\x16GYROSCOPE_UNCALIBRATED\x10\n\"\xd2\x01\n\nLogMessage\x12\x10\n\x08\x63ontents\x18\x01 \x01(\t\x12\r\n\x05start\x18\x02 \x01(\x03\x12\x0c\n\x04next\x18\x03 \x01(\x03\x12;\n\x04sort\x18\x04 \x01(\x0e\x32-.android.emulation.control.LogMessage.LogType\x12\x37\n\x07\x65ntries\x18\x05 \x03(\x0b\x32&.android.emulation.control.LogcatEntry\"\x1f\n\x07LogType\x12\x08\n\x04Text\x10\x00\x12\n\n\x06Parsed\x10\x01\"\x86\x02\n\x0bLogcatEntry\x12\x11\n\ttimestamp\x18\x01 \x01(\x04\x12\x0b\n\x03pid\x18\x02 \x01(\r\x12\x0b\n\x03tid\x18\x03 \x01(\r\x12>\n\x05level\x18\x04 \x01(\x0e\x32/.android.emulation.control.LogcatEntry.LogLevel\x12\x0b\n\x03tag\x18\x05 \x01(\t\x12\x0b\n\x03msg\x18\x06 \x01(\t\"p\n\x08LogLevel\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x0b\n\x07\x44\x45\x46\x41ULT\x10\x01\x12\x0b\n\x07VERBOSE\x10\x02\x12\t\n\x05\x44\x45\x42UG\x10\x03\x12\x08\n\x04INFO\x10\x04\x12\x08\n\x04WARN\x10\x05\x12\x07\n\x03\x45RR\x10\x06\x12\t\n\x05\x46\x41TAL\x10\x07\x12\n\n\x06SILENT\x10\x08\"\xf0\x01\n\x0fVmConfiguration\x12S\n\x0ehypervisorType\x18\x01 \x01(\x0e\x32;.android.emulation.control.VmConfiguration.VmHypervisorType\x12\x18\n\x10numberOfCpuCores\x18\x02 \x01(\x05\x12\x14\n\x0cramSizeBytes\x18\x03 \x01(\x03\"X\n\x10VmHypervisorType\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x08\n\x04NONE\x10\x01\x12\x07\n\x03KVM\x10\x02\x12\x08\n\x04HAXM\x10\x03\x12\x07\n\x03HVF\x10\x04\x12\x08\n\x04WHPX\x10\x05\x12\x07\n\x03GVM\x10\x06\"\x18\n\x08\x43lipData\x12\x0c\n\x04text\x18\x01 \x01(\t\"\xfa\x01\n\x05Touch\x12\t\n\x01x\x18\x01 \x01(\x05\x12\t\n\x01y\x18\x02 \x01(\x05\x12\x12\n\nidentifier\x18\x03 \x01(\x05\x12\x10\n\x08pressure\x18\x04 \x01(\x05\x12\x13\n\x0btouch_major\x18\x05 \x01(\x05\x12\x13\n\x0btouch_minor\x18\x06 \x01(\x05\x12\x44\n\nexpiration\x18\x07 \x01(\x0e\x32\x30.android.emulation.control.Touch.EventExpiration\"E\n\x0f\x45ventExpiration\x12 \n\x1c\x45VENT_EXPIRATION_UNSPECIFIED\x10\x00\x12\x10\n\x0cNEVER_EXPIRE\x10\x01\"P\n\nTouchEvent\x12\x31\n\x07touches\x18\x01 \x03(\x0b\x32 .android.emulation.control.Touch\x12\x0f\n\x07\x64isplay\x18\x02 \x01(\x05\"D\n\nMouseEvent\x12\t\n\x01x\x18\x01 \x01(\x05\x12\t\n\x01y\x18\x02 \x01(\x05\x12\x0f\n\x07\x62uttons\x18\x03 \x01(\x05\x12\x0f\n\x07\x64isplay\x18\x04 \x01(\x05\"\xc1\x02\n\rKeyboardEvent\x12\x46\n\x08\x63odeType\x18\x01 \x01(\x0e\x32\x34.android.emulation.control.KeyboardEvent.KeyCodeType\x12H\n\teventType\x18\x02 \x01(\x0e\x32\x35.android.emulation.control.KeyboardEvent.KeyEventType\x12\x0f\n\x07keyCode\x18\x03 \x01(\x05\x12\x0b\n\x03key\x18\x04 \x01(\t\x12\x0c\n\x04text\x18\x05 \x01(\t\"<\n\x0bKeyCodeType\x12\x07\n\x03Usb\x10\x00\x12\t\n\x05\x45vdev\x10\x01\x12\x07\n\x03XKB\x10\x02\x12\x07\n\x03Win\x10\x03\x12\x07\n\x03Mac\x10\x04\"4\n\x0cKeyEventType\x12\x0b\n\x07keydown\x10\x00\x12\t\n\x05keyup\x10\x01\x12\x0c\n\x08keypress\x10\x02\"2\n\x0b\x46ingerprint\x12\x12\n\nisTouching\x18\x01 \x01(\x08\x12\x0f\n\x07touchId\x18\x02 \x01(\x05\"\x8c\x01\n\x08GpsState\x12\x15\n\rpassiveUpdate\x18\x01 \x01(\x08\x12\x10\n\x08latitude\x18\x02 \x01(\x01\x12\x11\n\tlongitude\x18\x03 \x01(\x01\x12\r\n\x05speed\x18\x04 \x01(\x01\x12\x0f\n\x07\x62\x65\x61ring\x18\x05 \x01(\x01\x12\x10\n\x08\x61ltitude\x18\x06 \x01(\x01\x12\x12\n\nsatellites\x18\x07 \x01(\x05\"\x87\x04\n\x0c\x42\x61tteryState\x12\x12\n\nhasBattery\x18\x01 \x01(\x08\x12\x11\n\tisPresent\x18\x02 \x01(\x08\x12G\n\x07\x63harger\x18\x03 \x01(\x0e\x32\x36.android.emulation.control.BatteryState.BatteryCharger\x12\x13\n\x0b\x63hargeLevel\x18\x04 \x01(\x05\x12\x45\n\x06health\x18\x05 \x01(\x0e\x32\x35.android.emulation.control.BatteryState.BatteryHealth\x12\x45\n\x06status\x18\x06 \x01(\x0e\x32\x35.android.emulation.control.BatteryState.BatteryStatus\"W\n\rBatteryStatus\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x0c\n\x08\x43HARGING\x10\x01\x12\x0f\n\x0b\x44ISCHARGING\x10\x02\x12\x10\n\x0cNOT_CHARGING\x10\x03\x12\x08\n\x04\x46ULL\x10\x04\"9\n\x0e\x42\x61tteryCharger\x12\x08\n\x04NONE\x10\x00\x12\x06\n\x02\x41\x43\x10\x01\x12\x07\n\x03USB\x10\x02\x12\x0c\n\x08WIRELESS\x10\x03\"P\n\rBatteryHealth\x12\x08\n\x04GOOD\x10\x00\x12\n\n\x06\x46\x41ILED\x10\x01\x12\x08\n\x04\x44\x45\x41\x44\x10\x02\x12\x0f\n\x0bOVERVOLTAGE\x10\x03\x12\x0e\n\nOVERHEATED\x10\x04\"\xae\x01\n\x0eImageTransport\x12K\n\x07\x63hannel\x18\x01 \x01(\x0e\x32:.android.emulation.control.ImageTransport.TransportChannel\x12\x0e\n\x06handle\x18\x02 \x01(\t\"?\n\x10TransportChannel\x12!\n\x1dTRANSPORT_CHANNEL_UNSPECIFIED\x10\x00\x12\x08\n\x04MMAP\x10\x01\"P\n\rFoldedDisplay\x12\r\n\x05width\x18\x01 \x01(\r\x12\x0e\n\x06height\x18\x02 \x01(\r\x12\x0f\n\x07xOffset\x18\x03 \x01(\r\x12\x0f\n\x07yOffset\x18\x04 \x01(\r\"\xe5\x02\n\x0bImageFormat\x12@\n\x06\x66ormat\x18\x01 \x01(\x0e\x32\x30.android.emulation.control.ImageFormat.ImgFormat\x12\x35\n\x08rotation\x18\x02 \x01(\x0b\x32#.android.emulation.control.Rotation\x12\r\n\x05width\x18\x03 \x01(\r\x12\x0e\n\x06height\x18\x04 \x01(\r\x12\x0f\n\x07\x64isplay\x18\x05 \x01(\r\x12<\n\ttransport\x18\x06 \x01(\x0b\x32).android.emulation.control.ImageTransport\x12?\n\rfoldedDisplay\x18\x07 \x01(\x0b\x32(.android.emulation.control.FoldedDisplay\".\n\tImgFormat\x12\x07\n\x03PNG\x10\x00\x12\x0c\n\x08RGBA8888\x10\x01\x12\n\n\x06RGB888\x10\x02\"\x97\x01\n\x05Image\x12\x36\n\x06\x66ormat\x18\x01 \x01(\x0b\x32&.android.emulation.control.ImageFormat\x12\x11\n\x05width\x18\x02 \x01(\rB\x02\x18\x01\x12\x12\n\x06height\x18\x03 \x01(\rB\x02\x18\x01\x12\r\n\x05image\x18\x04 \x01(\x0c\x12\x0b\n\x03seq\x18\x05 \x01(\r\x12\x13\n\x0btimestampUs\x18\x06 \x01(\x04\"\xd5\x01\n\x08Rotation\x12\x42\n\x08rotation\x18\x01 \x01(\x0e\x32\x30.android.emulation.control.Rotation.SkinRotation\x12\r\n\x05xAxis\x18\x02 \x01(\x01\x12\r\n\x05yAxis\x18\x03 \x01(\x01\x12\r\n\x05zAxis\x18\x04 \x01(\x01\"X\n\x0cSkinRotation\x12\x0c\n\x08PORTRAIT\x10\x00\x12\r\n\tLANDSCAPE\x10\x01\x12\x14\n\x10REVERSE_PORTRAIT\x10\x02\x12\x15\n\x11REVERSE_LANDSCAPE\x10\x03\"\xf4\x01\n\tPhoneCall\x12\x41\n\toperation\x18\x01 \x01(\x0e\x32..android.emulation.control.PhoneCall.Operation\x12\x0e\n\x06number\x18\x02 \x01(\t\"\x93\x01\n\tOperation\x12\x0c\n\x08InitCall\x10\x00\x12\x0e\n\nAcceptCall\x10\x01\x12\x16\n\x12RejectCallExplicit\x10\x02\x12\x12\n\x0eRejectCallBusy\x10\x03\x12\x12\n\x0e\x44isconnectCall\x10\x04\x12\x13\n\x0fPlaceCallOnHold\x10\x05\x12\x13\n\x0fTakeCallOffHold\x10\x06\"\xbc\x01\n\rPhoneResponse\x12\x43\n\x08response\x18\x01 \x01(\x0e\x32\x31.android.emulation.control.PhoneResponse.Response\"f\n\x08Response\x12\x06\n\x02OK\x10\x00\x12\x10\n\x0c\x42\x61\x64Operation\x10\x01\x12\r\n\tBadNumber\x10\x02\x12\x11\n\rInvalidAction\x10\x03\x12\x10\n\x0c\x41\x63tionFailed\x10\x04\x12\x0c\n\x08RadioOff\x10\x05\"#\n\x05\x45ntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"<\n\tEntryList\x12/\n\x05\x65ntry\x18\x01 \x03(\x0b\x32 .android.emulation.control.Entry\"\xbd\x01\n\x0e\x45mulatorStatus\x12\x0f\n\x07version\x18\x01 \x01(\t\x12\x0e\n\x06uptime\x18\x02 \x01(\x04\x12\x0e\n\x06\x62ooted\x18\x03 \x01(\x08\x12<\n\x08vmConfig\x18\x04 \x01(\x0b\x32*.android.emulation.control.VmConfiguration\x12<\n\x0ehardwareConfig\x18\x05 \x01(\x0b\x32$.android.emulation.control.EntryList\"\xfe\x01\n\x0b\x41udioFormat\x12\x14\n\x0csamplingRate\x18\x01 \x01(\x04\x12\x41\n\x08\x63hannels\x18\x02 \x01(\x0e\x32/.android.emulation.control.AudioFormat.Channels\x12\x43\n\x06\x66ormat\x18\x03 \x01(\x0e\x32\x33.android.emulation.control.AudioFormat.SampleFormat\"/\n\x0cSampleFormat\x12\x0e\n\nAUD_FMT_U8\x10\x00\x12\x0f\n\x0b\x41UD_FMT_S16\x10\x01\" \n\x08\x43hannels\x12\x08\n\x04Mono\x10\x00\x12\n\n\x06Stereo\x10\x01\"g\n\x0b\x41udioPacket\x12\x36\n\x06\x66ormat\x18\x01 \x01(\x0b\x32&.android.emulation.control.AudioFormat\x12\x11\n\ttimestamp\x18\x02 \x01(\x04\x12\r\n\x05\x61udio\x18\x03 \x01(\x0c\".\n\nSmsMessage\x12\x12\n\nsrcAddress\x18\x01 \x01(\t\x12\x0c\n\x04text\x18\x02 \x01(\t\"\xcb\x02\n\x14\x44isplayConfiguration\x12\r\n\x05width\x18\x01 \x01(\r\x12\x0e\n\x06height\x18\x02 \x01(\r\x12\x0b\n\x03\x64pi\x18\x03 \x01(\r\x12\r\n\x05\x66lags\x18\x04 \x01(\r\x12\x0f\n\x07\x64isplay\x18\x05 \x01(\r\"\xe6\x01\n\x0c\x44isplayFlags\x12\x1c\n\x18\x44ISPLAYFLAGS_UNSPECIFIED\x10\x00\x12\x1f\n\x1bVIRTUAL_DISPLAY_FLAG_PUBLIC\x10\x01\x12%\n!VIRTUAL_DISPLAY_FLAG_PRESENTATION\x10\x02\x12\x1f\n\x1bVIRTUAL_DISPLAY_FLAG_SECURE\x10\x04\x12)\n%VIRTUAL_DISPLAY_FLAG_OWN_CONTENT_ONLY\x10\x08\x12$\n VIRTUAL_DISPLAY_FLAG_AUTO_MIRROR\x10\x10\"Z\n\x15\x44isplayConfigurations\x12\x41\n\x08\x64isplays\x18\x01 \x03(\x0b\x32/.android.emulation.control.DisplayConfiguration\"\xc8\x01\n\x0cNotification\x12@\n\x05\x65vent\x18\x01 \x01(\x0e\x32\x31.android.emulation.control.Notification.EventType\"v\n\tEventType\x12!\n\x1dVIRTUAL_SCENE_CAMERA_INACTIVE\x10\x00\x12\x1f\n\x1bVIRTUAL_SCENE_CAMERA_ACTIVE\x10\x01\x12%\n!DISPLAY_CONFIGURATIONS_CHANGED_UI\x10\x02\"1\n\x0eRotationRadian\x12\t\n\x01x\x18\x01 \x01(\x02\x12\t\n\x01y\x18\x02 \x01(\x02\x12\t\n\x01z\x18\x03 \x01(\x02\"+\n\x08Velocity\x12\t\n\x01x\x18\x01 \x01(\x02\x12\t\n\x01y\x18\x02 \x01(\x02\x12\t\n\x01z\x18\x03 \x01(\x02\"\xe8\x01\n\x07Posture\x12>\n\x05value\x18\x03 \x01(\x0e\x32/.android.emulation.control.Posture.PostureValue\"\x9c\x01\n\x0cPostureValue\x12\x13\n\x0fPOSTURE_UNKNOWN\x10\x00\x12\x12\n\x0ePOSTURE_CLOSED\x10\x01\x12\x17\n\x13POSTURE_HALF_OPENED\x10\x02\x12\x12\n\x0ePOSTURE_OPENED\x10\x03\x12\x13\n\x0fPOSTURE_FLIPPED\x10\x04\x12\x10\n\x0cPOSTURE_TENT\x10\x05\x12\x0f\n\x0bPOSTURE_MAX\x10\x06\x32\x8a\x18\n\x12\x45mulatorController\x12\x62\n\x0cstreamSensor\x12&.android.emulation.control.SensorValue\x1a&.android.emulation.control.SensorValue\"\x00\x30\x01\x12]\n\tgetSensor\x12&.android.emulation.control.SensorValue\x1a&.android.emulation.control.SensorValue\"\x00\x12M\n\tsetSensor\x12&.android.emulation.control.SensorValue\x1a\x16.google.protobuf.Empty\"\x00\x12[\n\x10setPhysicalModel\x12-.android.emulation.control.PhysicalModelValue\x1a\x16.google.protobuf.Empty\"\x00\x12r\n\x10getPhysicalModel\x12-.android.emulation.control.PhysicalModelValue\x1a-.android.emulation.control.PhysicalModelValue\"\x00\x12w\n\x13streamPhysicalModel\x12-.android.emulation.control.PhysicalModelValue\x1a-.android.emulation.control.PhysicalModelValue\"\x00\x30\x01\x12M\n\x0csetClipboard\x12#.android.emulation.control.ClipData\x1a\x16.google.protobuf.Empty\"\x00\x12M\n\x0cgetClipboard\x12\x16.google.protobuf.Empty\x1a#.android.emulation.control.ClipData\"\x00\x12R\n\x0fstreamClipboard\x12\x16.google.protobuf.Empty\x1a#.android.emulation.control.ClipData\"\x00\x30\x01\x12O\n\nsetBattery\x12\'.android.emulation.control.BatteryState\x1a\x16.google.protobuf.Empty\"\x00\x12O\n\ngetBattery\x12\x16.google.protobuf.Empty\x1a\'.android.emulation.control.BatteryState\"\x00\x12G\n\x06setGps\x12#.android.emulation.control.GpsState\x1a\x16.google.protobuf.Empty\"\x00\x12G\n\x06getGps\x12\x16.google.protobuf.Empty\x1a#.android.emulation.control.GpsState\"\x00\x12S\n\x0fsendFingerprint\x12&.android.emulation.control.Fingerprint\x1a\x16.google.protobuf.Empty\"\x00\x12M\n\x07sendKey\x12(.android.emulation.control.KeyboardEvent\x1a\x16.google.protobuf.Empty\"\x00\x12L\n\tsendTouch\x12%.android.emulation.control.TouchEvent\x1a\x16.google.protobuf.Empty\"\x00\x12L\n\tsendMouse\x12%.android.emulation.control.MouseEvent\x1a\x16.google.protobuf.Empty\"\x00\x12]\n\tsendPhone\x12$.android.emulation.control.PhoneCall\x1a(.android.emulation.control.PhoneResponse\"\x00\x12\\\n\x07sendSms\x12%.android.emulation.control.SmsMessage\x1a(.android.emulation.control.PhoneResponse\"\x00\x12P\n\tgetStatus\x12\x16.google.protobuf.Empty\x1a).android.emulation.control.EmulatorStatus\"\x00\x12[\n\rgetScreenshot\x12&.android.emulation.control.ImageFormat\x1a .android.emulation.control.Image\"\x00\x12`\n\x10streamScreenshot\x12&.android.emulation.control.ImageFormat\x1a .android.emulation.control.Image\"\x00\x30\x01\x12\x61\n\x0bstreamAudio\x12&.android.emulation.control.AudioFormat\x1a&.android.emulation.control.AudioPacket\"\x00\x30\x01\x12Q\n\x0binjectAudio\x12&.android.emulation.control.AudioPacket\x1a\x16.google.protobuf.Empty\"\x00(\x01\x12[\n\tgetLogcat\x12%.android.emulation.control.LogMessage\x1a%.android.emulation.control.LogMessage\"\x00\x12`\n\x0cstreamLogcat\x12%.android.emulation.control.LogMessage\x1a%.android.emulation.control.LogMessage\"\x00\x30\x01\x12M\n\nsetVmState\x12%.android.emulation.control.VmRunState\x1a\x16.google.protobuf.Empty\"\x00\x12M\n\ngetVmState\x12\x16.google.protobuf.Empty\x1a%.android.emulation.control.VmRunState\"\x00\x12\x80\x01\n\x18setDisplayConfigurations\x12\x30.android.emulation.control.DisplayConfigurations\x1a\x30.android.emulation.control.DisplayConfigurations\"\x00\x12\x66\n\x18getDisplayConfigurations\x12\x16.google.protobuf.Empty\x1a\x30.android.emulation.control.DisplayConfigurations\"\x00\x12Y\n\x12streamNotification\x12\x16.google.protobuf.Empty\x1a\'.android.emulation.control.Notification\"\x00\x30\x01\x12_\n\x18rotateVirtualSceneCamera\x12).android.emulation.control.RotationRadian\x1a\x16.google.protobuf.Empty\"\x00\x12^\n\x1dsetVirtualSceneCameraVelocity\x12#.android.emulation.control.Velocity\x1a\x16.google.protobuf.Empty\"\x00\x12J\n\nsetPosture\x12\".android.emulation.control.Posture\x1a\x16.google.protobuf.Empty\"\x00\x42&\n\x1c\x63om.android.emulator.controlP\x01\xa2\x02\x03\x41\x45\x43\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'android_env.proto.emulator_controller_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\034com.android.emulator.controlP\001\242\002\003AEC'
  _PARAMETERVALUE.fields_by_name['data']._options = None
  _PARAMETERVALUE.fields_by_name['data']._serialized_options = b'\020\001'
  _IMAGE.fields_by_name['width']._options = None
  _IMAGE.fields_by_name['width']._serialized_options = b'\030\001'
  _IMAGE.fields_by_name['height']._options = None
  _IMAGE.fields_by_name['height']._serialized_options = b'\030\001'
  _VMRUNSTATE._serialized_start=104
  _VMRUNSTATE._serialized_end=319
  _VMRUNSTATE_RUNSTATE._serialized_start=182
  _VMRUNSTATE_RUNSTATE._serialized_end=319
  _PARAMETERVALUE._serialized_start=321
  _PARAMETERVALUE._serialized_end=355
  _PHYSICALMODELVALUE._serialized_start=358
  _PHYSICALMODELVALUE._serialized_end=937
  _PHYSICALMODELVALUE_STATE._serialized_start=583
  _PHYSICALMODELVALUE_STATE._serialized_end=668
  _PHYSICALMODELVALUE_PHYSICALTYPE._serialized_start=671
  _PHYSICALMODELVALUE_PHYSICALTYPE._serialized_end=937
  _SENSORVALUE._serialized_start=940
  _SENSORVALUE._serialized_end=1444
  _SENSORVALUE_STATE._serialized_start=583
  _SENSORVALUE_STATE._serialized_end=668
  _SENSORVALUE_SENSORTYPE._serialized_start=1230
  _SENSORVALUE_SENSORTYPE._serialized_end=1444
  _LOGMESSAGE._serialized_start=1447
  _LOGMESSAGE._serialized_end=1657
  _LOGMESSAGE_LOGTYPE._serialized_start=1626
  _LOGMESSAGE_LOGTYPE._serialized_end=1657
  _LOGCATENTRY._serialized_start=1660
  _LOGCATENTRY._serialized_end=1922
  _LOGCATENTRY_LOGLEVEL._serialized_start=1810
  _LOGCATENTRY_LOGLEVEL._serialized_end=1922
  _VMCONFIGURATION._serialized_start=1925
  _VMCONFIGURATION._serialized_end=2165
  _VMCONFIGURATION_VMHYPERVISORTYPE._serialized_start=2077
  _VMCONFIGURATION_VMHYPERVISORTYPE._serialized_end=2165
  _CLIPDATA._serialized_start=2167
  _CLIPDATA._serialized_end=2191
  _TOUCH._serialized_start=2194
  _TOUCH._serialized_end=2444
  _TOUCH_EVENTEXPIRATION._serialized_start=2375
  _TOUCH_EVENTEXPIRATION._serialized_end=2444
  _TOUCHEVENT._serialized_start=2446
  _TOUCHEVENT._serialized_end=2526
  _MOUSEEVENT._serialized_start=2528
  _MOUSEEVENT._serialized_end=2596
  _KEYBOARDEVENT._serialized_start=2599
  _KEYBOARDEVENT._serialized_end=2920
  _KEYBOARDEVENT_KEYCODETYPE._serialized_start=2806
  _KEYBOARDEVENT_KEYCODETYPE._serialized_end=2866
  _KEYBOARDEVENT_KEYEVENTTYPE._serialized_start=2868
  _KEYBOARDEVENT_KEYEVENTTYPE._serialized_end=2920
  _FINGERPRINT._serialized_start=2922
  _FINGERPRINT._serialized_end=2972
  _GPSSTATE._serialized_start=2975
  _GPSSTATE._serialized_end=3115
  _BATTERYSTATE._serialized_start=3118
  _BATTERYSTATE._serialized_end=3637
  _BATTERYSTATE_BATTERYSTATUS._serialized_start=3409
  _BATTERYSTATE_BATTERYSTATUS._serialized_end=3496
  _BATTERYSTATE_BATTERYCHARGER._serialized_start=3498
  _BATTERYSTATE_BATTERYCHARGER._serialized_end=3555
  _BATTERYSTATE_BATTERYHEALTH._serialized_start=3557
  _BATTERYSTATE_BATTERYHEALTH._serialized_end=3637
  _IMAGETRANSPORT._serialized_start=3640
  _IMAGETRANSPORT._serialized_end=3814
  _IMAGETRANSPORT_TRANSPORTCHANNEL._serialized_start=3751
  _IMAGETRANSPORT_TRANSPORTCHANNEL._serialized_end=3814
  _FOLDEDDISPLAY._serialized_start=3816
  _FOLDEDDISPLAY._serialized_end=3896
  _IMAGEFORMAT._serialized_start=3899
  _IMAGEFORMAT._serialized_end=4256
  _IMAGEFORMAT_IMGFORMAT._serialized_start=4210
  _IMAGEFORMAT_IMGFORMAT._serialized_end=4256
  _IMAGE._serialized_start=4259
  _IMAGE._serialized_end=4410
  _ROTATION._serialized_start=4413
  _ROTATION._serialized_end=4626
  _ROTATION_SKINROTATION._serialized_start=4538
  _ROTATION_SKINROTATION._serialized_end=4626
  _PHONECALL._serialized_start=4629
  _PHONECALL._serialized_end=4873
  _PHONECALL_OPERATION._serialized_start=4726
  _PHONECALL_OPERATION._serialized_end=4873
  _PHONERESPONSE._serialized_start=4876
  _PHONERESPONSE._serialized_end=5064
  _PHONERESPONSE_RESPONSE._serialized_start=4962
  _PHONERESPONSE_RESPONSE._serialized_end=5064
  _ENTRY._serialized_start=5066
  _ENTRY._serialized_end=5101
  _ENTRYLIST._serialized_start=5103
  _ENTRYLIST._serialized_end=5163
  _EMULATORSTATUS._serialized_start=5166
  _EMULATORSTATUS._serialized_end=5355
  _AUDIOFORMAT._serialized_start=5358
  _AUDIOFORMAT._serialized_end=5612
  _AUDIOFORMAT_SAMPLEFORMAT._serialized_start=5531
  _AUDIOFORMAT_SAMPLEFORMAT._serialized_end=5578
  _AUDIOFORMAT_CHANNELS._serialized_start=5580
  _AUDIOFORMAT_CHANNELS._serialized_end=5612
  _AUDIOPACKET._serialized_start=5614
  _AUDIOPACKET._serialized_end=5717
  _SMSMESSAGE._serialized_start=5719
  _SMSMESSAGE._serialized_end=5765
  _DISPLAYCONFIGURATION._serialized_start=5768
  _DISPLAYCONFIGURATION._serialized_end=6099
  _DISPLAYCONFIGURATION_DISPLAYFLAGS._serialized_start=5869
  _DISPLAYCONFIGURATION_DISPLAYFLAGS._serialized_end=6099
  _DISPLAYCONFIGURATIONS._serialized_start=6101
  _DISPLAYCONFIGURATIONS._serialized_end=6191
  _NOTIFICATION._serialized_start=6194
  _NOTIFICATION._serialized_end=6394
  _NOTIFICATION_EVENTTYPE._serialized_start=6276
  _NOTIFICATION_EVENTTYPE._serialized_end=6394
  _ROTATIONRADIAN._serialized_start=6396
  _ROTATIONRADIAN._serialized_end=6445
  _VELOCITY._serialized_start=6447
  _VELOCITY._serialized_end=6490
  _POSTURE._serialized_start=6493
  _POSTURE._serialized_end=6725
  _POSTURE_POSTUREVALUE._serialized_start=6569
  _POSTURE_POSTUREVALUE._serialized_end=6725
  _EMULATORCONTROLLER._serialized_start=6728
  _EMULATORCONTROLLER._serialized_end=9810
# @@protoc_insertion_point(module_scope)
